//LICENSE A

//
//  A ViewSubject that represents a fixed set of solid surfaces (iActor)
//
#ifndef IACTORVIEWSUBJECT_H
#define IACTORVIEWSUBJECT_H


#include "iviewsubject.h"


#include "iposition.h"

class iActor;
class iActorViewSubject;


namespace iParameter
{
	namespace ViewObject
	{
		namespace Flag
		{
			const unsigned int HasNoColor =		 2U;
			const unsigned int HasNoOpacity =	 4U;
			const unsigned int IsSameColor =	 8U;
			const unsigned int IsSameOpacity =	16U;
		};
	};
};


class iActorViewInstance : public iViewInstance
{
	
	friend class iActorViewSubject;

public:
	
	iDataConsumerTypeMacroPass(iActorViewInstance,iViewInstance);

	iObjectSlotMacro1SR(Color,iColor);
	iObjectSlotMacro1SV(Opacity,float);

protected:
	
	iActorViewInstance(iActorViewSubject *owner, int num_actors, float max_opacity);
	virtual ~iActorViewInstance();

	iActorViewSubject* Owner() const;

	virtual void ShowBody(bool s);
	virtual void ResetBody();

	const float mMaxOpacity;
	iArray<iActor*> mActors;
};


class iActorViewSubject : public iViewSubject
{

	friend class iActorViewInstance;
	friend class iObjectFactory;

public:
	
	iDataConsumerTypeMacroPass(iActorViewSubject,iViewSubject);

	const bool HasNoColor, HasNoOpacity;
	const bool IsSameColor, IsSameOpacity;

	iType::vsp_color Color;
	iType::vsp_float Opacity;
		
	inline const iColor& GetDefaultColor() const { return mDefaultColor; }

protected:
	
	iActorViewSubject(iObject *parent, const iString &fname, const iString &sname, iViewModule *vm, const iDataType &type, unsigned int flags, int minsize, int maxsize);
	virtual ~iActorViewSubject();

	iColor mDefaultColor;
};


class iPropPlacementHelper
{

public:

	virtual bool SetBoxSize(double p);
	double GetBoxSize() const;

	void SetSize(double p);
	void SetSize(const iDistance &p);
	inline double GetSize() const { return mSize; }

	virtual bool SetDirection(const iVector3D& pos);
	const iVector3D& GetDirection() const { return mDirection; }

	virtual bool SetBoxPosition(const iVector3D& pos);
	const iVector3D& GetBoxPosition() const;

	void SetPosition(const iPosition &pos);
	inline const iPosition& GetPosition() const { return mPosition; }

protected:
	
	iPropPlacementHelper(iViewModule *vm);
	virtual ~iPropPlacementHelper();

	virtual void UpdateDirection() = 0;
	virtual void UpdatePosition() = 0;
	virtual void UpdateSize() = 0;

	iVector3D mDirection;
	iPosition mPosition;
	iDistance mSize;
};

#endif // IACTORVIEWSUBJECT_H

