//LICENSE A


#ifndef IARRAY_H
#define IARRAY_H

//
//  Automatically-resizable array of type T; the type must have operator =.
//

template<class T>
class iArray
{

public:

	iArray(int inc = 10);
	virtual ~iArray();

    T& operator[](int i);
    const T& operator[](int i) const;

	inline int Size() const { return this->mNum; }
	inline int MaxIndex() const { return this->mNum-1; }

	virtual void Add(const T &val);
	void Remove(int n);
	void Clear();
	void Resize(int n);

	inline T& Last(){ return this->mArr[this->mNum-1]; }
    inline const T& Last() const { return this->mArr[this->mNum-1]; }
	void AddLast();
	T RemoveLast();  // useful for deleteting all: while(arr.Size() > 0) delete arr.RemoveLast();

	inline const T* Data() const { return this->mArr; }

protected:

	void Extend(int len);

	T* mArr;
	int mNum, mLen;
	const int mInc;

private:

	iArray(const iArray<T>&); // not implemented
	void operator=(const iArray<T>&); // not implemented
};


//
//  iLookupArray class
//  Needs operator == for type T
//
template<class T>
class iLookupArray : public iArray<T>
{

public:

	iLookupArray(int inc = 10);

	virtual int Find(const T &val) const;
	bool AddUnique(const T &val);
	bool Remove(const T &val);

private:

};


//
//  iGenericOrderedArray class
//  Needs operator < for the key type K
//
template<class T, class K>
class iGenericOrderedArray : public iLookupArray<T>
{

public:

	iGenericOrderedArray(const K& (*key)(const T &val), int inc = 10);

	//
	//  In case of duplicate keys, add the new value last
	//
	virtual void Add(const T &val);
	virtual int Find(const T &val) const;
	//
	//  In case of duplicate keys, return the first value
	//
	int FindByKey(const K &key) const;

private:

	const K& (*Key)(const T &val);

	iGenericOrderedArray(const iGenericOrderedArray<T,K>&); // not implemented
	void operator=(const iGenericOrderedArray<T,K>&); // not implemented
};


//
//  iOrderedArray class (order by its own value)
//
template<class T>
class iOrderedArray : public iGenericOrderedArray<T,T>
{

public:

	iOrderedArray(int inc = 10) : iGenericOrderedArray<T,T>(Self,inc){}

private:

	static const T& Self(const T &val){ return val; }

	iOrderedArray(const iOrderedArray<T>&); // not implemented
	void operator=(const iOrderedArray<T>&); // not implemented
};

#endif // IARRAY_H
