//LICENSE A


#include "ibitmaptextsubject.h"


#include "ierror.h"
#include "ioverlayhelper.h"
#include "itextactor.h"

#include <vtkActor2D.h>
#include <vtkTextMapper.h>
#include <vtkTextProperty.h>
#include <vtkViewport.h>

//
//  Templates
//
#include "iarray.tlh"
#include "igenericprop.tlh"


iBitmapTextSubject* iBitmapTextSubject::New(iTextActor *parent, iRenderTool *rt)
{
	IASSERT(rt);
	return new iBitmapTextSubject(parent,rt);
}


iBitmapTextSubject::iBitmapTextSubject(iTextActor *parent, iRenderTool *rt) : iTextSubject(parent,rt)
{
	mMapper = vtkTextMapper::New(); IERROR_CHECK_MEMORY(mMapper);
	mActor = vtkActor2D::New(); IERROR_CHECK_MEMORY(mActor);
	this->AppendComponent(mActor);
	mActor->SetMapper(mMapper);
	mActor->GetPositionCoordinate()->SetCoordinateSystemToNormalizedViewport();

	mMapper->SetTextProperty(mParent->GetTextProperty()); 
}


iBitmapTextSubject::~iBitmapTextSubject()
{
	mMapper->Delete();
	mActor->Delete();
}


void iBitmapTextSubject::ComputeSize(vtkViewport *viewport, float s[2])
{
	mMapper->SetInput(mParent->GetText().ToCharPointer());

	double w = mMapper->GetWidth(viewport);
	double h = mMapper->GetHeight(viewport);
	viewport->ViewportToNormalizedViewport (w,h);

	s[0] = w;
	s[1] = h;
}


void iBitmapTextSubject::UpdateGeometryBody(vtkViewport* viewport, int mag)
{
	if(mag == 1)
	{
		//mMapper->GetTextProperty()->SetOrientation(mParent->GetAngle());
		mActor->SetPosition(mPos[0],mPos[1]);
		mActor->SetPosition2(mSize[0],mSize[1]);
	}
	else
	{
		int winij[2];
		this->GetOverlayHelper()->ComputePositionShiftsUnderMagnification(winij);
		
		mActor->SetPosition(mag*mPos[0]-winij[0],mag*mPos[1]-winij[1]);
		mActor->SetPosition2(mag*mSize[0],mag*mSize[1]);
	}
}

