//LICENSE A


#include "iboundeddisksource.h"


#include "ierror.h"

#include <vtkAppendPolyData.h>
#include <vtkArrowSource.h>
#include <vtkDiskSource.h>
#include <vtkPolyData.h>
#include <vtkPolyDataNormals.h>
#include <vtkTransform.h>
#include <vtkTransformPolyDataFilter.h>

//
//  Templates
//
#include "igenericfilter.tlh"


//
// iBoundedDiskSource class
//
iBoundedDiskSource* iBoundedDiskSource::New(iDataConsumer *consumer)
{
	IASSERT(consumer);
	return new iBoundedDiskSource(consumer);
}


iBoundedDiskSource::iBoundedDiskSource(iDataConsumer *consumer) : iBoundedPolyDataSource(consumer,true,true)
{
	mPointNormals = vtkPolyDataNormals::New(); IERROR_CHECK_MEMORY(mPointNormals);
	mSource = vtkDiskSource::New(); IERROR_CHECK_MEMORY(mSource);

	mPointNormals->SplittingOff();
	mPointNormals->ConsistencyOff();
	mPointNormals->ComputePointNormalsOn();
	mPointNormals->ComputeCellNormalsOff();
	mPointNormals->FlipNormalsOff();

	mSource->SetInnerRadius(0.0);
	mSource->SetOuterRadius(1.0);

	mAppendPolyData = vtkAppendPolyData::New(); IERROR_CHECK_MEMORY(mAppendPolyData);
	mArrowSource = vtkArrowSource::New(); IERROR_CHECK_MEMORY(mArrowSource);
	mTransformPolyDataFilter = vtkTransformPolyDataFilter::New(); IERROR_CHECK_MEMORY(mTransformPolyDataFilter);
	vtkTransform *t = vtkTransform::New(); IERROR_CHECK_MEMORY(t);
	t->Identity();
	t->RotateY(-90.0);
	mTransformPolyDataFilter->SetTransform(t);
	t->Delete();
	mTransformPolyDataFilter->SetInputConnection(mArrowSource->GetOutputPort());
	mAppendPolyData->AddInputConnection(mSource->GetOutputPort());
	mAppendPolyData->AddInputConnection(mTransformPolyDataFilter->GetOutputPort());

	mWithArrow = false;
	this->SetWithArrow(true);
}


iBoundedDiskSource::~iBoundedDiskSource()
{
	mSource->Delete();
	mPointNormals->Delete();
	mAppendPolyData->Delete();
	mArrowSource->Delete();
	mTransformPolyDataFilter->Delete();
}


void iBoundedDiskSource::SetWithArrow(bool s)
{
	if(s == mWithArrow) return;

	mWithArrow = s;
	if(s)
	{
		mFilter->SetInputConnection(mAppendPolyData->GetOutputPort());
	}
	else
	{
		mFilter->SetInputConnection(mSource->GetOutputPort());
	}
}


void iBoundedDiskSource::AddObserverToSource(unsigned long e, vtkCommand *c, float p)
{
	mSource->AddObserver(e,c,p);
	mPointNormals->AddObserver(e,c,p);
}


void iBoundedDiskSource::UpdateSourceResolution()
{
	mSource->SetRadialResolution(1+mResolution/2);
	mSource->SetCircumferentialResolution(3*mResolution);
}


float iBoundedDiskSource::GetSourceMemorySize() const
{
	float s = 0.0;
	s += mPointNormals->GetOutput()->GetActualMemorySize();
	s += mSource->GetOutput()->GetActualMemorySize();
	return s;
}


void iBoundedDiskSource::UpdateBoundaryConditions()
{
}

