//LICENSE A


#ifndef IBOUNDEDPOLYDATASOURCE_H
#define IBOUNDEDPOLYDATASOURCE_H


#include "igenericfilter.h"
#include <vtkPolyDataAlgorithm.h>


class iLimits;
class iTransform;

class vtkClipPolyData;
class vtkCommand;
class vtkPlanes;
class vtkTransformPolyDataFilter;


class iBoundedPolyDataSource : public iGenericPolyDataFilter<vtkPolyDataAlgorithm>
{

public:
	
	vtkTypeMacro(iBoundedPolyDataSource,vtkPolyDataAlgorithm);

	unsigned long AddObserver(unsigned long e, vtkCommand *c, float priority = 0.0);
	
	void SetCenter(const double x[3]); 
	inline const double* GetCenter() const { return mCenter; }
		
	void SetResolution(int r); 
	inline int GetResolution() const { return mResolution; }
		
	void SetBounds(double b0, double b1, double b2, double b3, double b4, double b5); 
	inline const double* GetBounds() const { return mBounds; }

	void SetRadius(double r);
	inline double GetRadius() const { return mRadius; }

	void SetNormal(const double n[3]);
	inline const double* GetNormal() const { return mNormal; }

	float GetMemorySize() const;

protected:
	
	iBoundedPolyDataSource(iDataConsumer *consumer, bool normalCanChange, bool radiusCanChange);
	virtual ~iBoundedPolyDataSource();

	virtual void ProvideOutput();

	virtual void UpdateBoundaryConditions() = 0;
	virtual void UpdateSourceResolution() = 0;
	virtual void AddObserverToSource(unsigned long e, vtkCommand *c, float priority) = 0;
	virtual float GetSourceMemorySize() const = 0;

	double mBounds[6];
	double mCenter[3], mNormal[3], mRadius;
	int mResolution;

	bool mNormalChanged;

	const bool mNormalCanChange, mRadiusCanChange;

	//
	//  VTK stuff
	//
	iTransform *mTransform;
	vtkPlanes *mClipPlanes;
	vtkClipPolyData *mClipper;
	vtkTransformPolyDataFilter *mFilter;
};

#endif // IBOUNDEDPOLYDATASOURCE_H

