//LICENSE A


#include "iboundedspheresource.h"


#include "idataconsumer.h"
#include "idatasubject.h"
#include "ierror.h"
#include "ispheresource.h"
#include "iviewmodule.h"

#include <vtkAppendPolyData.h>
#include <vtkPolyData.h>
#include <vtkTransformPolyDataFilter.h>


//
// iBoundedSphereSource class
//
iBoundedSphereSource* iBoundedSphereSource::New(iDataConsumer *consumer)
{
	IASSERT(consumer);
	return new iBoundedSphereSource(consumer);
}


iBoundedSphereSource::iBoundedSphereSource(iDataConsumer *consumer) : iBoundedPolyDataSource(consumer,false,true)
{
	mAppend = vtkAppendPolyData::New(); IERROR_CHECK_MEMORY(mAppend);
	for(int i=0; i<8; i++)
	{
		mSource[i] = iSphereSource::New(); IERROR_CHECK_MEMORY(mSource[i]);
	}
	mAppend->UserManagedInputsOn();
	mAppend->SetInputConnection(mSource[0]->GetOutputPort());

	mSource[0]->SetResolution(1);
	mSource[0]->SetRadius(1.0);

	mFilter->SetInputConnection(mSource[0]->GetOutputPort());
}


iBoundedSphereSource::~iBoundedSphereSource()
{
	mAppend->Delete();
	for(int i=0; i<8; i++)
	{
		mSource[i]->Delete();
	}
}


void iBoundedSphereSource::AddObserverToSource(unsigned long e, vtkCommand *c, float p)
{
	mAppend->AddObserver(e,c,p);
	for(int i=0; i<8; i++)
	{
		mSource[i]->AddObserver(e,c,p);
	}
}


void iBoundedSphereSource::UpdateSourceResolution()
{
	mSource[0]->SetResolution(mResolution);
}


float iBoundedSphereSource::GetSourceMemorySize() const
{
	float s = 0.0;
	for(int i=0; i<8; i++)
	{
		s += mSource[i]->GetOutput()->GetActualMemorySize();
	}
	s += mAppend->GetOutput()->GetActualMemorySize();
	return s;
}


void iBoundedSphereSource::UpdateBoundaryConditions()
{
	int i, j, l, side[3];
	double cen[3], xb;

	//
	//  Test for out-of-bounds pieces
	//
	for(i=0; i<3; i++)
	{
		side[i] = 0;
		if(this->Consumer()->GetSubject()->IsDirectionPeriodic(i))
		{
			xb = mCenter[i] - mRadius;
			if(xb < mBounds[2*i+0]) side[i] = -1;  //  we should only cross one boundary
			xb = mCenter[i] + mRadius;
			if(xb > mBounds[2*i+1]) side[i] =  1;
		}
	}

	if(side[0]==0 && side[1]==0 && side[2]==0)
	{
		//
		//  Fully inside
		//
		mFilter->SetInputConnection(mSource[0]->GetOutputPort());
		return;
	}

	mFilter->SetInputConnection(mAppend->GetOutputPort());

	l = 1;
	for(i=0; i<3; i++)
	{
		if(side[i] != 0)
		{
			mAppend->SetNumberOfInputs(2*l);
			for(j=l; j<2*l; j++)
			{
				mSource[j]->SetRadius(1.0);
				mSource[j]->SetResolution(mResolution);
				mSource[j-l]->GetCenter(cen);
				cen[i] -= 2.0*side[i]/mRadius;
				mSource[j]->SetCenter(cen);
#ifdef IVTK_5
				mAppend->SetInputByNumber(j,mSource[j]->GetOutput());
#else
				mAppend->SetInputConnectionByNumber(j,mSource[j]->GetOutputPort());
#endif
			}
			l *= 2;
		}
	}
}






