//LICENSE A

//
//  This class is a merger of iObject with vtkCamera class. But since we cannot inherit, we must decorate
//
#ifndef ICAMERA_H
#define ICAMERA_H


#include "iobject.h"
#include "iviewmodulecomponent.h"


#include "iposition.h"
#include "ipropertyposition.h"

class iEventObserver;
class iRenderTool;

class vtkCamera;


class iCamera : public iObject, public iViewModuleComponent
{

	friend class iExtensionFactory;

public:

	vtkTypeMacro(iCamera,iObject);
	static iCamera* New(iRenderTool *rt = 0);

	iObjectSlotMacro2S(Position,const iPosition&);
	iObjectSlotMacro2S(FocalPoint,const iPosition&);
	iPropertyScalarPosition Position, FocalPoint;

	//
	//  Camera orientation
	//
	iObjectPropertyMacro2S(ViewUp,Vector);
	iObjectPropertyMacro2S(ClippingRange,Pair);
	iObjectPropertyMacro2S(ParallelProjection,Bool);
	iObjectPropertyMacro2S(ParallelScale,Double);
	iObjectPropertyMacro2S(ViewAngle,Double);
	iObjectPropertyMacro2S(ViewAngleVertical,Bool);
	iObjectPropertyMacro2S(EyeAngle,Double);
	iObjectPropertyMacro2S(ClippingRangeAuto,Bool);

	//
	//  actions
	//
	iObjectPropertyMacroA1(Azimuth,Double);
	iObjectPropertyMacroA1(Elevation,Double);
	iObjectPropertyMacroA1(Yaw,Double);
	iObjectPropertyMacroA1(Pitch,Double);
	iObjectPropertyMacroA1(Roll,Double);
	iObjectPropertyMacroA1(Zoom,Double);
	iObjectPropertyMacroA(ResetCamera);
	iObjectPropertyMacroA(OrthogonalizeCameraView);
	
	bool Reset();
	bool OrthogonalizeView();
	void ShiftTo(const iPosition& pos);

	vtkCamera* GetDevice() const { return mDevice; }

protected:

	virtual ~iCamera();

private:

	iCamera(iRenderTool *rt, const iString &fname, const iString &sname);

	void SyncProjections();

	bool mSyncProjections;
	vtkCamera *mDevice;
	iRenderTool *mRenderTool;

	mutable iVector3D mViewUp;
	mutable iPair mClippingRange;
	mutable iPosition mPosition, mFocalPoint;
};

#endif

