//LICENSE A

//
//  DataSubject(s) for core IFrIT DataType(s)
//
#ifndef ICOREDATASUBJECTS_H
#define ICOREDATASUBJECTS_H


#include "iparticledatasubject.h"

class iDataReader;


namespace iCoreData
{
	class ScalarFileLoader;


	class ScalarDataSubject : public iDataSubject
	{

		friend class ::iDataReader;

	public:

		vtkTypeMacro(ScalarDataSubject,iDataSubject);

		iObjectPropertyMacroA2(Smooth,Int,Float);

		//
		//  Inherited members
		//
		static const iDataType& DataType();
		virtual const iDataType& GetDataType() const;

		//
		//  Data subjects are treated by help differently from other objects
		//
		virtual const iString HelpTag() const;

	private:

		ScalarDataSubject(ScalarFileLoader *fl); // should not be created except by a Reader

		ScalarFileLoader *mConcreteLoader;
	};


	class VectorDataSubject : public iDataSubject
	{

		friend class ::iDataReader;

	public:

		vtkTypeMacro(VectorDataSubject,iDataSubject);

		//
		//  Inherited members
		//
		static const iDataType& DataType();
		virtual const iDataType& GetDataType() const;

		//
		//  Data subjects are treated by help differently from other objects
		//
		virtual const iString HelpTag() const;

	protected:

		virtual iDataLimits* CreateLimits() const;

	private:

		VectorDataSubject(iFileLoader *fl); // should not be created except by a Reader
	};


	class TensorDataSubject : public iDataSubject
	{

		friend class ::iDataReader;

	public:

		vtkTypeMacro(TensorDataSubject,iDataSubject);

		//
		//  Inherited members
		//
		static const iDataType& DataType();
		virtual const iDataType& GetDataType() const;

		//
		//  Data subjects are treated by help differently from other objects
		//
		virtual const iString HelpTag() const;

	protected:

		virtual iDataLimits* CreateLimits() const;

	private:

		TensorDataSubject(iFileLoader *fl); // should not be created except by a Reader
	};


	class ParticleDataSubject : public iParticleDataSubject
	{

		friend class ::iDataReader;

	public:

		vtkTypeMacro(ParticleDataSubject,iParticleDataSubject);

		//
		//  Inherited members
		//
		static const iDataType& DataType();
		virtual const iDataType& GetDataType() const;

		//
		//  Data subjects are treated by help differently from other objects
		//
		virtual const iString HelpTag() const;

	private:

		ParticleDataSubject(iParticleFileLoader *fl); // should not be created except by a Reader
	};
};

#endif

