//LICENSE A


//
//  Header file for iCrossSectionViewSubject class
//
#ifndef ICROSSSECTIONVIEWSUBJECT_H
#define ICROSSSECTIONVIEWSUBJECT_H


#include "imultiviewsubject.h"


class iCrossSectionViewSubject;
class iViewObject;

class vtkImageData;
class vtkTexture;


namespace iParameter
{
	namespace SpecialLocation
	{
		const int MaxData =	-4;
		const int MinData =	-3;
	};
};


class iCrossSectionViewInstance : public iMultiViewInstance
{
	
	friend class iCrossSectionViewSubject;

public:
	
	iDataConsumerTypeMacro(iCrossSectionViewInstance,iMultiViewInstance);
		
	iObjectSlotMacro1SV(Method,int);
	iObjectSlotMacro1SV(Direction,int);
	iObjectSlotMacro1SV(SampleRate,int);
	iObjectSlotMacro1SV(InterpolateData,bool);

	virtual bool SetBoxLocation(double p);
	double GetBoxLocation() const;

	void SetLocation(double p);
	void SetLocation(const iCoordinate &p);
	inline double GetLocation() const { return mLocation[mDirection]; }
	
	bool SetToSpecialLocation(int n);

	inline vtkImageData* GetTextureData() const { return mTextureData; }
	
	inline bool GetOverTheEdgeFlag() const { return mOverTheEdgeFlag; }
	int GetActualMethod() const;

	virtual float GetMemorySize();
	virtual void RemoveInternalData();	

protected:
	
	iCrossSectionViewInstance(iCrossSectionViewSubject *owner);
	virtual ~iCrossSectionViewInstance();
	virtual void ConfigureBody();
	virtual void FinishInitialization();
	virtual void ConfigureMainPipeline(iViewSubjectPipeline *p, int id);

	virtual void ShowBody(bool s);
	virtual bool CanBeShown() const;

	virtual void UpdatePainting();

	virtual iViewSubjectPipeline* CreatePipeline(int id);
	void UpdateLocation();
	void UpdateTextureSize();
	
	bool mOverTheEdgeFlag;
	iPosition mLocation;
	//
	//  VTK stuff
	//
	vtkTexture *mTexture;
	vtkImageData *mTextureData;
};


class iCrossSectionViewSubject : public iMultiViewSubject 
{
	
	friend class iCrossSectionViewInstance;

public:
	
	iViewSubjectTypeMacro(iCrossSectionView,iMultiViewSubject);

	iType::vsp_int Method;
	iType::vsp_int Direction;
	iType::vsp_int SampleRate;
	iType::vsp_bool InterpolateData;
	iType::vsp_double Location;

	iCrossSectionViewInstance* GetInstance(int i = 0) const;
	static void ForcePolygonalMethod(bool s);

protected:
	
	iCrossSectionViewSubject(iViewObject *parent, const iDataType &type);
	virtual ~iCrossSectionViewSubject();

	virtual void UpdateAutomaticShadingBody();

	static bool mForcePolygonalMethod;
};

#endif // ICROSSSECTIONVIEWSUBJECT_H

