//LICENSE A


#include "idatareaderobserver.h"


#include "idata.h"
#include "ierror.h"
#include "ioutput.h"
#include "istring.h"


using namespace iParameter;


//
//  iDataReaderObserver class
//
iDataReaderObserver::iDataReaderObserver(iShell *s) : iShellComponent(s)
{
	mStarted = false;
}


void iDataReaderObserver::StartSet()
{
	if(mStarted)
	{
		IBUG_ERROR("iDataReaderObserver cannot be called recursively.");
		return;
	}
	mStarted = true;
	this->OnSetStart();
}


void iDataReaderObserver::StartFile(const iString &fname, const iDataType &type)
{
	this->OnFileStart(fname,type);
}


void iDataReaderObserver::FinishFile(const iString &fname, const iDataInfo &info)
{
	this->OnFileFinish(fname,info);
}


void iDataReaderObserver::FinishSet(bool error)
{
	this->OnSetFinish(error);
	mStarted = false;
}


bool iDataReaderObserver::AllowReload()
{
	this->OutputText(MessageType::Warning,"This property only takes effect when the data set is reloaded. Use <DataReader:Reload()> function call to reload the data as needed.");
	return false;
}



void iDataReaderObserver::OnSetStart()
{
}


void iDataReaderObserver::OnFileFinish(const iString &fname, const iDataInfo &info)
{
}


void iDataReaderObserver::OnSetFinish(bool error)
{
}


void iDataReaderObserver::OnFileStart(const iString &fname, const iDataType &type)
{
	this->OutputText(MessageType::Information,"Loading file: "+fname+" of data type "+type.LongName());
}

