//LICENSE A

//
//  A class that accesses file names in a directory. It uses vtkDirectory
//  class internally, but changes interface to more convenient.
//

#ifndef IDIRECTORY_H
#define IDIRECTORY_H


#include "iarray.h"
#include "istring.h"


class iDirectory
{

public:

	iDirectory();

	bool Open(const iString &name);
	void Close();
	inline bool IsOpened() const { return !mDirectory.IsEmpty(); }
	inline const iString& GetDirectoryPrefix() const { return mDirectory; }
	
	inline int GetNumberOfFiles() const { return mFiles.Size(); }
	const iString GetFile(int i, bool prefixed = false) const;
	const iString GetFile(const iString & file, int skip) const;
 
	bool Make(const iString &name);

	//
	//  Directory naming convention for portability
	//
	static const iString& Separator();
	static const iString& Current();
	static void ExpandFileName(iString& name, const iString &path = "");

private:

	iString mDirectory;
	iOrderedArray<iString> mFiles;
};

#endif // IDIRECTORY_H

