//LICENSE A

//
//  Some useful error reporting macros and function(s)
//

#ifndef IERROR_H
#define IERROR_H


#include "ioutput.h"


#define IBUG_WARN(_m_)				{ iOutput::ReportBug(_m_,__FILE__,__LINE__,2); }
#define IBUG_ERROR(_m_)				{ iOutput::ReportBug(_m_,__FILE__,__LINE__,1); }
#define IBUG_FATAL(_m_)				{ iOutput::ReportBug(_m_,__FILE__,__LINE__,0); }
#define IASSERT(_p_)				{ if((_p_) == 0) IBUG_FATAL("Invalid use of a null pointer."); }
#define IERROR_CHECK_MEMORY(_p_)	{ if((_p_) == 0) iOutput::ReportBug("There is not enough memory to proceed.",__FILE__,__LINE__,-1); }

//
//  error-reporting dynamic_cast - make it a global function to reduce de-referencing
//  (ideally we would avoid using it all - use iRequiredCast(INFO,vtkObject*) function whenever possible).
//
template<class O, class I>
inline O* iDynamicCast(int line, const char *file, I* p)
{
	if(p == 0) return 0;
	O* tmp = dynamic_cast<O*>(p);
	if(tmp == 0) iOutput::ReportBug("Invalid cast.",file,line,0);
	return tmp;
}

//
//  RequiredCast for children of vtkObjectBase (which must be all cast-able objects)
//
class vtkObjectBase;
template<class O>
inline O* iRequiredCast(int line, const char *file, vtkObjectBase *p)
{
	if(p == 0) return 0;
	O* tmp = O::SafeDownCast(p);
	if(tmp == 0) iOutput::ReportBug("Invalid cast.",file,line,0);
	return tmp;
}

#define INFO __LINE__,__FILE__

#endif  // IERROR_H

