//LICENSE A


#include "iextendableobject.h"


#include "idata.h"

//
//  Templates
//
#include "iarray.tlh"
#include "iproperty.tlh"


//
//  Helper class
//
iObjectExtension::iObjectExtension(iExtendableObject *owner, const iString &fname, const iString &sname) : iObject(owner,fname,sname)
{
	IASSERT(owner);

	mOwner = owner;
	mRenderMode = owner->GetRenderMode();
}


iObjectExtension::~iObjectExtension()
{
}


//
//  Main class
//
iExtendableObject::iExtendableObject(iObject *parent, const iString &fname, const iString &sname) : iObject(parent,fname,sname)
{
}


iExtendableObject::~iExtendableObject()
{
	while(mExtensions.Size() > 0) mExtensions.RemoveLast()->Delete();
}


iObjectExtension* iExtendableObject::GetExtensionByName(const iString &name) const
{
	int i;
	for(i=0; i<mExtensions.Size(); i++) if(mExtensions[i]->LongName() == name) return mExtensions[i];
	return 0;
}


iObjectExtension* iExtendableObject::GetExtensionByClassName(const iString &name) const
{
	int i;
	for(i=0; i<mExtensions.Size(); i++) if(mExtensions[i]->IsA(name.ToCharPointer())) return mExtensions[i];
	return 0;
}


void iExtendableObject::InstallExtension(iObjectExtension *object)
{
	IASSERT(object);
	mExtensions.AddUnique(object);

	//
	//  We don't treat extensions as children
	//
	mChildren.Remove(object);

	//
	//  Register extension properties
	//
	int j;
	for(j=0; j<object->mVariableProperties.Size(); j++)
	{
		mVariableProperties.Add(object->mVariableProperties[j]);
	}
	for(j=0; j<object->mFunctionProperties.Size(); j++)
	{
		mFunctionProperties.Add(object->mFunctionProperties[j]);
	}
}
