//LICENSE A


#include "ifieldglyphpipeline.h"


#include "idatalimits.h"
#include "ifieldviewsubject.h"
#include "ireplicatedpolydata.h"
#include "iresampleimagedatafilter.h"

//
//  Templates (needed for some compilers)
//
#include "iarray.tlh"
#include "igenericfilter.tlh"
#include "iviewsubjectpipeline.tlh"


iPipelineKeyDefineMacro(iFieldGlyphPipeline,GlyphSize);
iPipelineKeyDefineMacro(iFieldGlyphPipeline,GlyphSampleRate);

	
//
//  generic iFieldGlyphPipeline class
//
iFieldGlyphPipeline::iFieldGlyphPipeline(iFieldViewInstance *owner) : iReplicatedViewSubjectPipeline(owner,1)
{
	mOwner = owner;

	//
	//  Do VTK stuff
	//	
	mResampleFilter = this->CreateFilter<iResampleImageDataFilter>();
}


iFieldGlyphPipeline::~iFieldGlyphPipeline()
{
}


bool iFieldGlyphPipeline::PrepareInput()
{
	vtkDataSet *input = this->InputData();

	if(mResampleFilter!=0 && input->IsA("vtkImageData"))  // this is so that child classes can circumvent the resample filter by deleting it.
	{
		vtkImageData *inp = vtkImageData::SafeDownCast(input);
		if(mResampleFilter->InputData() != inp)
		{
			mResampleFilter->SetInputData(inp);
		}
	}
	else
	{
		this->SetGlyphFilterInput(input);
	}

	//
	//  Needed so that the glyph size is correct
	//
	this->UpdateGlyphSize();

	return true;
}


void iFieldGlyphPipeline::UpdateGlyphSize()
{
	//
	//  Do not update unless necessary
	//
	if(mOwner->CanBeShown())
	{
		double s = 10.0*mOwner->GetGlyphSize()*2.0/this->GetLimits()->GetMax(0); // 2.0 is the box size
		this->SetGlyphFilterScale(s);
		this->Modified();
	}
}


void iFieldGlyphPipeline::UpdateGlyphSampleRate()
{ 
	mResampleFilter->SetResampleRate(mOwner->GetGlyphSampleRate());
	this->Modified();
}

