//LICENSE A


#include "ifieldviewsubject.h"


#include "iactor.h"
#include "icolorbar.h"
#include "idatareader.h"
#include "idatasubject.h"
#include "imergedatafilter.h"
#include "iviewmodule.h"

#include <vtkDataSet.h>

//
//  Templates (needed for some compilers)
//
#include "iarray.tlh"
#include "igenericfilter.tlh"
#include "iproperty.tlh"


using namespace iParameter;


//
//  Single instance class
//
iFieldViewInstance::iFieldViewInstance(iFieldViewSubject *owner, int numactors) : iPaintedViewInstance(owner,numactors,1.0,1,ColorBarPriority::Field,false)
{
	mGlyphSize = 0.1;
	mGlyphSampleRate = 16;

	this->AddSecondaryDataType(owner->GetPaintingDataType());

	//
	//  Do VTK stuff
	//	
	mMergeDataFilter = iMergeDataFilter::New(this); IERROR_CHECK_MEMORY(mMergeDataFilter);
}


iFieldViewInstance::~iFieldViewInstance()
{
	mMergeDataFilter->Delete();
}


iFieldViewSubject* iFieldViewInstance::Owner() const
{
	return iRequiredCast<iFieldViewSubject>(INFO,this->iPaintedViewInstance::Owner());
}


bool iFieldViewInstance::SetGlyphSize(float v)
{ 
	if(v < 10.0)
	{
		mGlyphSize = v;
		this->UpdateGlyphSize();
		return true;
	}
	else return false;
}


bool iFieldViewInstance::SetGlyphSampleRate(int v)
{ 
	if(v>0 && v<1000) 
	{
		mGlyphSampleRate = v;
		this->UpdateGlyphSampleRate();
		return true;
	}
	else return false;
}


bool iFieldViewInstance::IsConnectedToPaintingData() const
{
	if(this->IsThereData())
	{
		return mMergeDataFilter->HasData(0);
	}
	else return true;
}


bool iFieldViewInstance::SyncWithDataBody()
{
	if(this->IsThereData())
	{
		mMergeDataFilter->SetInputData(1,this->GetViewModule()->GetReader()->GetData(this->Owner()->GetPaintingDataType()));
		mMergeDataFilter->SetInputData(0,this->GetSubject()->GetData());
		mMergeDataFilter->Update();
		this->ResetPipelineInput(mMergeDataFilter->OutputData());
	}

	return true;
}


iFieldViewSubject::iFieldViewSubject(iObject *parent, const iString &fname, const iString &sname, iViewModule *vm, const iDataType &type, const iDataType &sdt, unsigned int flags) : iPaintedViewSubject(parent,fname,sname,vm,type,sdt,flags,1,1),
	iViewSubjectPropertyConstructMacro(Float,iFieldViewInstance,GlyphSize,gs),
	iViewSubjectPropertyConstructMacro(Int,iFieldViewInstance,GlyphSampleRate,gr),
	iObjectConstructPropertyMacroQ(Bool,iFieldViewSubject,IsConnectedToScalars,cs)
{
	this->AddSecondaryDataType(mPaintingDataType);
}


iFieldViewSubject::~iFieldViewSubject()
{
}


bool iFieldViewSubject::GetIsConnectedToScalars() const
{
	int i;

	for(i=0; i<mInstances.Size(); i++)
	{
		if(!iRequiredCast<iFieldViewInstance>(INFO,mInstances[i])->IsConnectedToPaintingData()) return false;
	}

	return true;
}

