//LICENSE A


//
//  A front-end for portable props - in case VTK guys decide to change the vtkProp interface again...
//
#ifndef IGENERICPROP_H
#define IGENERICPROP_H


#include "iarray.h"
#include "ivtk.h"

#include <vtkTimeStamp.h>

class vtkProp;
class vtkViewport;
class vtkWindow;


template<class Prop>
class iGenericProp : public Prop
{

public:

	virtual int RenderOverlay(vtkViewport *vp);
	virtual int RenderOpaqueGeometry(vtkViewport *vp);
	virtual int HasTranslucentPolygonalGeometry(); 
	virtual int RenderTranslucentPolygonalGeometry(vtkViewport *vp);
	virtual int RenderVolumetricGeometry(vtkViewport *vp);

	virtual void ReleaseGraphicsResources(vtkWindow *win);

protected:

	iGenericProp(bool self);

	virtual void UpdateGeometry(vtkViewport *vp) = 0;
	virtual void UpdateOverlay(vtkViewport *vp);
	virtual void Reset();

	inline bool IsEnabled() const { return mEnabled; }
	void PrependComponent(vtkProp *p);
	void AppendComponent(vtkProp *p);
	void RemoveComponent(vtkProp *p);
	void RemoveAllComponents();
	void ToggleComponent(vtkProp *p);
	void ShowComponents(int n, vtkProp **p);
	void Disable();

	vtkTimeStamp GetLastRenderMTime() const { return mLastRenderMTime; }

private:

	void MakeUpTodate(vtkViewport *vp);

	const bool mSelf;
	bool mDirty, mEnabled;
	iLookupArray<vtkProp*> mProps;

	vtkTimeStamp mLastRenderMTime;
};

#endif // IGENERICPROP_H
 
