//LICENSE A


#include "ihistogram.h"


#include "ierror.h"
#include "imath.h"
#include "istretch.h"

#include <string.h>

//
//  Templates
//
#include "iarray.tlh"


iHistogram::iHistogram()
{
	this->Clear();
}


iHistogram::~iHistogram()
{
}


const iHistogram& iHistogram::Null()
{
	static iHistogram null;
	return null;
}


void iHistogram::Clear()
{
	mValues.Clear();
	mBins.Clear();
	mBins.Add(0);

	mStretchId = iStretch::Lin;
	mStretchedMin = 0;
	mStretchedMax = 0;
	mValueToBinScale = 0;
}


void iHistogram::Reset(int nbins, float min, float max, int sid)
{
	int i;

	if(nbins<1 || min>=max || sid<0 || sid>=iStretch::Count)
	{
		this->Clear();
		return;
	}

	mBins.Resize(nbins+1);
	mBins[0] = min;
	mBins[nbins] = max;
	mStretchId = sid;

	mStretchedMin = iStretch::Apply(min,sid,false);
	mStretchedMax = iStretch::Apply(max,sid,true);
	float s = (mStretchedMax-mStretchedMin)/nbins;
	for(i=1; i<nbins; i++)
	{
		mBins[i] = iStretch::Reset(mStretchedMin+s*i,sid);
	}
	mValueToBinScale = 0.99999f*nbins/(mStretchedMax-mStretchedMin);

	mValues.Resize(nbins);
	memset((void *)mValues.Data(),0,nbins*sizeof(float));
}


int iHistogram::Bin(float v) const
{
#ifdef I_CHECK
	this->Check();
#endif
	if(mValues.Size()==0 || v<mBins[0] || v>mBins[mValues.Size()]) return -1;
	return int(mValueToBinScale*(iStretch::Apply(v,mStretchId,false)-mStretchedMin));
}


float iHistogram::GetMin() const
{
#ifdef I_CHECK
	this->Check();
#endif
	return mBins[0];
}


float iHistogram::GetMax() const
{
#ifdef I_CHECK
	this->Check();
#endif
	return mBins[mValues.Size()];
}


void iHistogram::GetRange(float range[2]) const
{
#ifdef I_CHECK
	this->Check();
#endif
	range[0] = mBins[0];
	range[1] = mBins[mValues.Size()];
}


float iHistogram::GetBinMin(int i) const
{
#ifdef I_CHECK
	this->Check();
#endif
	if(i>=0 && i<mValues.Size())
	{
		return mBins[i];
	}
	else return 0.0;
}


float iHistogram::GetBinMax(int i) const
{
#ifdef I_CHECK
	this->Check();
#endif
	if(i>=0 && i<mValues.Size())
	{
		return mBins[i+1];
	}
	else return 0.0;
}


void iHistogram::GetBinRange(int i, float range[2]) const
{
#ifdef I_CHECK
	this->Check();
#endif
	if(i>=0 && i<mValues.Size())
	{
		range[0] = mBins[i];
		range[1] = mBins[i+1];
	}
}


float iHistogram::Value(int i) const
{
#ifdef I_CHECK
	this->Check();
#endif
	if(i>=0 && i<mValues.Size())
	{
		return mValues[i];
	}
	else return -1;
}


#ifdef I_CHECK
void iHistogram::Check() const
{
	if(mValues.Size()+1 != mBins.Size())
	{
		IBUG_FATAL("Incorrectly configure iHistogram");
	}
}
#endif
