//LICENSE A


#ifndef IIMAGE_H
#define IIMAGE_H


#include "istring.h"


class iColor;

class vtkImageData;


class iImage
{
	
public:

	//
	//  Fit both directions when scaling, or fit into the requested size/include the requested size
	//  while preserving the original axes ratio.
	//
	enum ScaleMode
	{
		_Both = 0,
		_Fit = 1,
		_Include = 2,
		_FitMin = 3
	};

	enum ScaleFilter
	{
		_None = 0,
		_Smooth = 1
	};
	
	iImage(int depth = 3);
	iImage(const iImage& img);
	~iImage();
	
	//
	//  Operators
	//
    iImage& operator=(const iImage &img);   // impl-shared copy
    friend bool operator==(const iImage &s1, const iImage &s2);
    friend bool operator!=(const iImage &s1, const iImage &s2);
	
	//
	//  Loading from file
	//
	static bool LoadFromFile(const iString &filename, iImage &image);
	bool LoadFromFile(const iString &filename);
	
	//
	//  Useful functions for manipulating images
	//
	void Scale(int w, int h, ScaleMode m = _Both, ScaleFilter f = _None);
	void Smooth(float dx, float dy);
	void Clear();
	void ReplaceData(vtkImageData *n);
	void Blend(const iImage& im, float op);
	void Overlay(int x, int y, const iImage& ovr, float opacity = 1.0, bool masking = true);
	bool CombineInPseudoColor(const iImage &imRed, const iImage &imGreen, const iImage &imBlue);
	void Crop(int x, int y, int w, int h);
	void DrawFrame(int width, const iColor &color);

	//
	//  Quering the data
	//
	inline bool IsEmpty() const { return mData == 0; }
	inline int Depth() const { return mDepth; }
	inline int Width() const { return mWidth; }
	inline int Height() const { return mHeight; }
	inline void Dimensions(int d[2]) const { d[0] = mWidth; d[1] = mHeight; }

	//
	//  Direct access to underlying data: need to be very careful with direct data access!!!
	//
	unsigned char* DataPointer() const;
	inline vtkImageData* DataObject() const { return mData; }
	void SetDataPointer(unsigned char* array, int w, int h, int d);

private:

	//
	//  Helper function: if depth is set to 0, then the data are not actually allocated
	//
	vtkImageData* CreateData(int w, int h, int depth = 0);

	vtkImageData *mData;
	int mDepth, mWidth, mHeight;
};


inline bool operator!=(const iImage &s1, const iImage &s2)
{
	return !(s1==s2);
}

#endif // IIMAGE_H

