//LICENSE A


#include "ilookuptable.h"


#include "ierror.h"
#include "imath.h"
#include "ipalette.h"
#include "ipalettecollection.h"
#include "istretch.h"

#include <vtkMath.h>


//
//  Optimized vtkLookupTable
//
iLookupTable* iLookupTable::New()
{
	return new iLookupTable;
}


iLookupTable::iLookupTable()
{
	mPaletteId = 0;
}


iLookupTable::~iLookupTable()
{
}


void iLookupTable::SetPaletteId(int n)
{
	if(n != mPaletteId)
	{
		mPaletteId = n;
		this->Modified();
		this->Build();
	}
}


void iLookupTable::SetReversed(bool s)
{
	if(s && mPaletteId>0)
	{
		mPaletteId = -mPaletteId;
		this->Modified();
		this->Build();
	}
	if(!s && mPaletteId<0)
	{
		mPaletteId = -mPaletteId;
		this->Modified();
		this->Build();
	}
}


void iLookupTable::SetColor(const iColor &c)
{
	double *rgb = c.ToVTK();
	double hsv[3];
	vtkMath::RGBToHSV(rgb,hsv);
	this->SetHueRange(hsv[0],hsv[0]);
	this->SetSaturationRange(hsv[1],hsv[1]);
	this->SetValueRange(0.0,1.0);
	this->Modified();
	this->Build();
}


void iLookupTable::SetStretchId(int s)
{
	switch(s)
	{
	case iStretch::Log:
		{
			if(this->TableRange[0] < 1*iMath::_DoubleRes) this->TableRange[0] = 1*iMath::_DoubleRes;
			if(this->TableRange[1] < 2*iMath::_DoubleRes) this->TableRange[1] = 2*iMath::_DoubleRes;
			this->SetScaleToLog10();
			break;
		}
	default:
		{
			this->SetScaleToLinear();
		}
	}
	this->Modified();
}


void iLookupTable::ForceBuild()
{
	vtkLookupTable *lut = iPaletteCollection::Global()->GetLookupTable(mPaletteId);

	if(lut != 0)
	{
		this->Table->Initialize();
		this->Table->SetArray(lut->GetPointer(0),4*lut->GetNumberOfTableValues(),1);
		this->BuildTime.Modified();
	}
	else
	{
		this->Table->Initialize();
		vtkLookupTable::ForceBuild();
	}
}


unsigned long iLookupTable::GetMTime()
{
	vtkLookupTable *lut = iPaletteCollection::Global()->GetLookupTable(mPaletteId);
	if(lut != 0)
	{
		if(lut->GetMTime() > this->vtkObject::GetMTime()) this->Modified();
	}

	return this->vtkObject::GetMTime();
}


void iLookupTable::MapScalarsThroughTable2(void *input, unsigned char *output, int inputDataType, int numberOfValues, int inputIncrement, int outputFormat)
{
	//
    // This is how this function is used.
	//
	//this->MapScalarsThroughTable2(scalars->GetVoidPointer(comp), 
    //                              newColors->GetPointer(0),
    //                              scalars->GetDataType(),
    //                              scalars->GetNumberOfTuples(),
    //                              scalars->GetNumberOfComponents(), 
    //                              VTK_RGBA);

	this->vtkLookupTable::MapScalarsThroughTable2(input,output,inputDataType,numberOfValues,inputIncrement,outputFormat);
}

