//LICENSE A


#ifndef IMEASURINGBOX_H
#define IMEASURINGBOX_H


#include "igenericprop.h"
#include "iactor.h"
#include "iviewmoduletool.h"


#include <vtkProperty.h>

class iColor;
class iMeasuringBox;
class iTextActor;

class vtkCommand;
class vtkCubeSource;
class vtkInteractorStyle;
class vtkPolyDataMapper;


class iMeasuringBoxActor : public iGenericProp<iActor>, public iViewModuleComponent
{
	
	friend class iMeasuringBox;

public:
	
	vtkTypeMacro(iMeasuringBoxActor,iActor);
	
	void AttachToInteractorStyle(vtkInteractorStyle *s);

	void SetSize(float s);
	void SetOpacity(float s);
	void SetColor(iColor &c);
	void SetBaseScale(float s);

	inline float GetOpacity(){ return mBoxActor->GetProperty()->GetOpacity(); }
	inline float GetSize(){ return mSize; }
	
	virtual double *GetBounds(){ return 0; }  // Set to zero so that the text is always displayed

protected:
	
	virtual ~iMeasuringBoxActor();
	
	virtual void UpdateGeometry(vtkViewport *vp);

private:

	iMeasuringBoxActor(iViewModule *vm);

	iTextActor *mText;
	iActor *mFrameActor, *mBoxActor, *mWorkerActor;

	vtkCommand *mObserver;

	bool mStarted;
	float mSize, mBaseScale;
	float mFactor1r, mFactor1g, mFactor1b, mFactor2;

};


class iMeasuringBox : public iViewModuleTool
{
	
public:

	vtkTypeMacro(iMeasuringBox,iViewModuleTool);
	static iMeasuringBox* New(iViewModule *vm = 0);
		
	inline iMeasuringBoxActor* GetActor() const { return mActor; }

protected:

	virtual ~iMeasuringBox();

private:
	
	iMeasuringBox(iViewModule *vm, const iString &fname, const iString &sname);

	virtual bool ShowBody(bool s);

	//
	//  Actors displayed by this class
	//
	iMeasuringBoxActor *mActor;
};

#endif // IMEASURINGBOX_H
