//LICENSE A


#include "imonitor.h"


#include "ierror.h"
#include "iobject.h"
#include "ishell.h"
#include "ishelleventobservers.h"


//
//  Convenience class
//
iMonitor::iMonitor(const iShellComponent *user)
{
	IASSERT(user);
	mObserver = user->GetShell()->GetExecutionEventObserver(); IASSERT(mObserver);
	mObserver->Start();
	mObserver->SetInterval(0.0,1.0);
}

iMonitor::iMonitor(const iShellComponent *user, const iString& label, double length)
{
	IASSERT(user);
	mObserver = user->GetShell()->GetExecutionEventObserver(); IASSERT(mObserver);
	mObserver->Start();
	mObserver->SetLabel(label);
	mObserver->SetInterval(0.0,length);
}


iMonitor::~iMonitor()
{
	IASSERT(mObserver);
	mObserver->Finish();
}


void iMonitor::SetInterval(double start, double length)
{
	IASSERT(mObserver);
	mObserver->SetInterval(start,length);
}


void iMonitor::SetInterval(const iString& label, double start, double length)
{
	IASSERT(mObserver);
	mObserver->SetLabel(label);
	mObserver->SetInterval(start,length);
}


void iMonitor::SetProgress(double prog)
{
	IASSERT(mObserver);
	mObserver->SetProgress(prog);
}


bool iMonitor::IsStopped() const
{
	IASSERT(mObserver);
	return (mObserver->IsAborted() || mObserver->IsInError());
}


bool iMonitor::IsAborted() const
{
	IASSERT(mObserver);
	return mObserver->IsAborted();
}


void iMonitor::PostError(const iString &text)
{
	IASSERT(mObserver);
	mObserver->PostError(text);
}


void iMonitor::Reset()
{
	IASSERT(mObserver);
	mObserver->Reset();
}


const iString& iMonitor::LastErrorMessage() const
{
	IASSERT(mObserver);
	return mObserver->LastErrorMessage();
}
