//LICENSE A


#include "iobjectcollection.h"


#include "ierror.h"
#include "ihelpfactory.h"
#include "istring.h"

//
//  Templates
//
#include "iarray.tlh"
#include "iproperty.tlh"


//
//  Main class
//
iObjectCollection::iObjectCollection(iObject *parent, const iString &fname, const iString &sname) : iObject(parent,fname,sname)
{
}


iObject* iObjectCollection::GetMember(int i) const
{
	if(i>=0 && i<mChildren.Size()) return mChildren[i]; else return 0;
}


bool iObjectCollection::AutoCreate = false;


const iObject* iObjectCollection::FindByName(const iString &name) const
{
	if(name==this->ShortName() || name==this->LongName()) return this;

	//
	//  Require an index for a member.
	//
	bool ok;
	iString prefix = name.Section(".", 0, 0);
	if (prefix.BeginsWith(this->ShortName() + "[") || prefix.BeginsWith(this->LongName() + "["))
	{
		if (!prefix.EndsWith("]")) return 0; // incorrect index
		int i = prefix.Find('[');
		iString sidx = prefix.Part(i, prefix.Length() - 1 - i);
		int idx = sidx.ToInt(ok);
		if (ok && idx >= 0 && idx < this->Size())
		{
			//
			//  There is an index specifier
			//
			const iObject *obj = this->GetMember(idx); IASSERT(obj);
			if (name.Contains(".") > 0) obj = obj->FindByName(name.Section(".", 1));
			return obj;
		}
		else return 0; // incorrect index
	}
	else return 0;
}


void iObjectCollection::RegisterWithHelpFactory() const
{
	this->RegisterPropertiesWithHelpFactory();

	int i;
	for(i=0; i<mChildren.Size(); i++)
	{
		mChildren[i]->RegisterWithHelpFactory();
	}
}

