//LICENSE A


#ifndef IOBJECTCOLLECTION_H
#define IOBJECTCOLLECTION_H


#include "iobject.h"


class iObjectCollection : public iObject
{

	friend class iShell;

public:

	vtkTypeMacro(iObjectCollection,iObject);

	inline int Size() const { return mChildren.Size(); }
	virtual iObject* GetMember(int i) const;

	virtual void RegisterWithHelpFactory() const;

	virtual const iObject* FindByName(const iString &name) const;

protected:

	virtual bool CreateMember() = 0;
	virtual bool RemoveMember(int i) = 0;

	iObjectCollection(iObject *parent, const iString &fname, const iString &sname);

	static bool AutoCreate;
};


//
//  A simple collection of the fixed number of iObjects
//
template<class Object>
class iObjectCollectionFixed : public iObjectCollection
{

public:

	iObjectPropertyMacro1S(Number,Int);

	vtkTypeMacro(iObjectCollectionFixed,iObjectCollection);

protected:

	iObjectCollectionFixed(iObject *parent, const iString &fname, const iString &sname, int minsize);
	virtual ~iObjectCollectionFixed();

	virtual bool CreateMember();
	virtual bool RemoveMember(int i);

	virtual Object* NewObject() = 0;

	const int MinSize;
};


//
//  Resizable collection
//
template<class Object>
class iObjectCollectionResizable : public iObjectCollectionFixed<Object>
{

public:

	vtkTypeMacro(iObjectCollectionResizable,iObjectCollectionFixed<Object>);

	iPropertyAction1<iType::Int> Remove;

protected:

	iObjectCollectionResizable(iObject *parent, const iString &fname, const iString &sname, int minsize);

	virtual bool RemoveMember(int i);
};

#endif // IOBJECTCOLLECTION_H
