//LICENSE A


#include "ioverlayhelper.h"


#include "ierror.h"
#include "imagnifier.h"
#include "imath.h"
#include "irendertool.h"

#include <vtkCamera.h>
#include <vtkProperty2D.h>
#include <vtkRenderer.h>
#include <vtkTextProperty.h>


using namespace iParameter;


iOverlayHelper* iOverlayHelper::New(iRenderTool *rt)
{
	IASSERT(rt);
	return new iOverlayHelper(rt);
}


iOverlayHelper::iOverlayHelper(iRenderTool *rt) : mRenderTool(rt)
{
	mIsAutoColor = true;
	mFontFactor = 1.0f;
}


iOverlayHelper::~iOverlayHelper()
{
}


const iColor& iOverlayHelper::GetColor(vtkViewport* viewport)
{
	if(mIsAutoColor && viewport!=0)
	{
		int i;
		double bg[3];
		viewport->GetBackground(bg);
		for(i=0; i<3; i++) bg[i] = 1.0 - bg[i];
		wColor = iColor(bg);
	}
	else
	{
		wColor = mFixedColor;
	}
	return wColor;
}


int iOverlayHelper::GetRenderingMagnification() const
{
	return mRenderTool->GetRenderingMagnification();
}


void iOverlayHelper::ComputePositionShiftsUnderMagnification(int winij[2])
{
	int mag = mRenderTool->GetRenderingMagnification();
	//
	//  Compute position shifts if under magnification
	//
	if(mag > 1)
	{
		mRenderTool->GetMagnifier()->GetTile(winij[0],winij[1]);
	}
	else winij[0] = winij[1] = 0;
}


vtkCamera* iOverlayHelper::GetCamera(vtkViewport *vp) const
{
	vtkRenderer *ren = vtkRenderer::SafeDownCast(vp);
	if(ren == 0) return 0; else return ren->GetActiveCamera();
}


int iOverlayHelper::GetFontSize(vtkViewport *vp, float factor) const
{
	float v = factor*16*mFontFactor*exp(0.3*mRenderTool->GetFontScale())/480;
	vtkRenderer *ren = vtkRenderer::SafeDownCast(vp);
	if(ren != 0)
	{
		if(ren->GetActiveCamera()->GetUseHorizontalViewAngle() != 0)
		{
			return iMath::Round2Int(v*vp->GetSize()[0]);
		}
		else
		{
			return iMath::Round2Int(v*vp->GetSize()[1]);
		}
	}
	else return iMath::Round2Int(v*480);
}


void iOverlayHelper::UpdateTextProperty(vtkViewport *vp, vtkTextProperty *prop)
{
#ifndef I_NO_CHECK
	int mag = mRenderTool->GetRenderingMagnification();
	if(mag > 1)
	{
		IBUG_WARN("Extra call to iOverlayHelper::UpdateTextProperty");
	}
#endif

	if(prop != 0)
	{
		switch(mRenderTool->GetFontType())
		{
		case TextType::Arial:
			{
				prop->SetFontFamilyToArial();
				mFontFactor = 1.0;
				break;
			}
		case TextType::Courier:
			{
				prop->SetFontFamilyToCourier();
				mFontFactor = 1.2;
				break;
			}
		case TextType::Times:
			{
				prop->SetFontFamilyToTimes();
				mFontFactor = 1.0;
				break;
			}
		}
		prop->ItalicOff();
		prop->ShadowOn();
		prop->SetColor(this->GetColor(vp).ToVTK());
	}
}

