//LICENSE A

//
//  A helper class for all 2D objects; includes basic functionality for working under
//  magnification.
//
#ifndef IOVERLAYHELPER_H
#define IOVERLAYHELPER_H


#include <vtkObjectBase.h>


#include "icolor.h"

#include "ipointermacro.h"

#include <vtkSetGet.h>

class iRenderTool;

class vtkCamera;
class vtkProperty2D;
class vtkTextProperty;
class vtkViewport;


class iOverlayHelper : public vtkObjectBase
{

	IPOINTER_AS_PART(RenderTool);

public:

	vtkTypeMacro(iOverlayHelper,vtkObjectBase);
	static iOverlayHelper* New(iRenderTool *rv = 0);

	int GetFontSize(vtkViewport *vp, float factor = 1.0f) const;
	void UpdateTextProperty(vtkViewport *vp, vtkTextProperty *prop);

	vtkCamera* GetCamera(vtkViewport *viewport) const;
	int GetRenderingMagnification() const;
	void ComputePositionShiftsUnderMagnification(int winij[2]);

	void SetFixedColor(const iColor &c){ mFixedColor = c; }
	inline const iColor& GetFixedColor() const { return mFixedColor; }
	const iColor& GetColor(vtkViewport* viewport);

	void SetAutoColor(bool s){ mIsAutoColor = s; }
	inline bool IsAutoColor() const { return mIsAutoColor; } 

protected:

	virtual ~iOverlayHelper();

private:
	
	iOverlayHelper(iRenderTool *rv);

	iColor mFixedColor;
	bool mIsAutoColor; 
	float mFontFactor;

	//
	//  Work variable
	//
	iColor wColor;
};

#endif // IOVERLAYHELPER_H
