//LICENSE A


#ifndef IPALETTE_H
#define IPALETTE_H


#include "iobject.h"


#include "icolor.h"
#include "ipair.h"
#include "istring.h"

class iImage;
class iPaletteCollection;
class iPiecewiseFunction;

class vtkColorTransferFunction;
class vtkLookupTable;


class iPalette : public iObject
{
	
	friend class iPaletteCollection;

public:
	
	vtkTypeMacro(iPalette,iObject);

	iObjectPropertyMacro1S(Name,String);
	iPropertyPiecewiseFunction Red, Green, Blue;

	virtual bool CopyState(const iObject *p);

	iColor GetColor(int n) const;
	const iImage* GetImage(int shape = 0);
	
	void SetComponents(const iPiecewiseFunction *r, const iPiecewiseFunction *g, const iPiecewiseFunction *b);
	iPiecewiseFunction* GetComponent(int n) const;

	inline vtkLookupTable* GetLookupTable(bool reverse) const { return mLT[reverse?1:0]; }
	inline vtkColorTransferFunction* GetColorTransferFunction(bool reverse) const { return mCTF[reverse?1:0]; }

	void Update();

protected:
	
	iPalette(iObject *parent, const iString &fname, const iString &sname);
	virtual ~iPalette();

private:

	iPiecewiseFunction *mRed, *mGreen, *mBlue;
	vtkColorTransferFunction *mCTF[2];
	vtkLookupTable *mLT[2];

	int mImageWidth, mImageHeight;
	bool mImageNeedsUpdate;
	iImage *mImage[3];
};

#endif // IPALETTE_H
