//LICENSE A


#include "iparticledatasubject.h"


#include "idata.h"
#include "idatalimits.h"
#include "ierror.h"
#include "iparticlefileloader.h"
#include "iparticleprobefilter.h"

//
//  Templates
//
#include "iarray.tlh"
#include "iproperty.tlh"


iParticleDataSubject::iParticleDataSubject(iParticleFileLoader *fl, const iDataType& type) : iDataSubject(fl,type), 
	iObjectConstructPropertyMacroQ(Bool,iParticleDataSubject,UseExtras,ue),
	iObjectConstructPropertyMacroS(Bool,iParticleDataSubject,TypeIncluded,ti),
	iObjectConstructPropertyMacroS(Int,iParticleDataSubject,DownsampleMode,dm),
	iObjectConstructPropertyMacroS(Int,iParticleDataSubject,DownsampleFactor,df),
	iObjectConstructPropertyMacroS(Bool,iParticleDataSubject,AddOrderAsVar,ov),
	iObjectConstructPropertyMacroS(Int,iParticleDataSubject,DensityVar,dv)
{	
	DensityVar.AddFlag(iProperty::_FunctionsAsIndex);

	mParticleLoader = fl; IASSERT(mParticleLoader);
}


iDataLimits* iParticleDataSubject::CreateLimits() const
{
	return iDataLimits::New(this);
}



bool iParticleDataSubject::GetTypeIncluded() const
{
	return mParticleLoader->GetTypeIncluded(this->GetId());
}


int iParticleDataSubject::GetDownsampleMode() const
{
	return mParticleLoader->GetDownsampleMode();
}


int iParticleDataSubject::GetDownsampleFactor() const
{
	return mParticleLoader->GetDownsampleFactor(this->GetId());
}


bool iParticleDataSubject::GetAddOrderAsVar() const
{
	return mParticleLoader->GetAddOrderAsVariable(this->GetId());
}


int iParticleDataSubject::GetDensityVar() const
{
	return mParticleLoader->GetDensityVariable(this->GetId());
}


bool iParticleDataSubject::GetUseExtras() const
{
	return mParticleLoader->IsUsingExtras();
}


bool iParticleDataSubject::SetTypeIncluded(bool b)
{
	mParticleLoader->SetTypeIncluded(this->GetId(),b);
	return true;;
}


bool iParticleDataSubject::SetDownsampleMode(int i)
{
	mParticleLoader->SetDownsampleMode(i);
	return true;;
}


bool iParticleDataSubject::SetDownsampleFactor(int i)
{
	mParticleLoader->SetDownsampleFactor(this->GetId(),i);
	return true;;
}


bool iParticleDataSubject::SetAddOrderAsVar(bool b)
{
	mParticleLoader->SetAddOrderAsVariable(this->GetId(),b);
	return true;;
}


bool iParticleDataSubject::SetDensityVar(int i)
{
	mParticleLoader->SetDensityVariable(this->GetId(),i);
	return true;;
}


iProbeFilter* iParticleDataSubject::CreateProbeFilter(iDataConsumer *consumer) const
{
	return iParticleProbeFilter::New(consumer);
}

