//LICENSE A

//
//  A base class for all DataSubjects of particle types.
//
#ifndef IPARTICLEDATASUBJECT_H
#define IPARTICLEDATASUBJECT_H


#include "idatasubject.h"


class iParticleFileLoader;


class iParticleDataSubject : public iDataSubject
{

	friend class iParticleFileLoader;

public:

	vtkTypeMacro(iParticleDataSubject,iDataSubject);

	iObjectPropertyMacro2Q(UseExtras,Bool);
	iObjectPropertyMacro2S(TypeIncluded,Bool);
	iObjectPropertyMacro2S(DownsampleMode,Int);
	iObjectPropertyMacro2S(DownsampleFactor,Int);
	iObjectPropertyMacro2S(AddOrderAsVar,Bool);
	iObjectPropertyMacro2S(DensityVar,Int);

	virtual iProbeFilter* CreateProbeFilter(iDataConsumer *consumer) const;

protected:

	iParticleDataSubject(iParticleFileLoader *fl, const iDataType& type);

	virtual iDataLimits* CreateLimits() const;

private:

	iParticleFileLoader *mParticleLoader;
};


//
//  Useful macros to declare all members that have to be overwritten in children
//
#define iParticleDataSubjectDeclareClassMacro(_prefix_,_name_) \
class _prefix_##_name_##DataSubject : public iParticleDataSubject \
{ \
public: \
	vtkTypeMacro(_prefix_##_name_##DataSubject,iParticleDataSubject); \
	static const iDataType& DataType(); \
	const iDataType& GetDataType() const; \
	_prefix_##_name_##DataSubject(iParticleFileLoader *fl); \
}

#define iParticleDataSubjectDefineClassMacro(_prefix_,_name_,_fname_,_sname_,_keywords_,_environment_) \
iDataSubjectDefineTypeMacro(_prefix_##_name_##DataSubject,_prefix_##Extension::SubjectId(),_fname_,_sname_,0,_keywords_,_environment_); \
_prefix_##_name_##DataSubject::_prefix_##_name_##DataSubject(iParticleFileLoader *fl) : iParticleDataSubject(fl,_prefix_##_name_##DataSubject::DataType()){}


#endif

