//LICENSE A

//
//  Helper class for computing particle density in place (NOT a filter!!!) - there is no elegant way to add
//  a variable without replicating the data in the memory.
//  It can be inherited to implement a faster density assignment scheme.
//
#ifndef IPARTICLEDENSITYESTIMATOR_H
#define IPARTICLEDENSITYESTIMATOR_H


#include <vtkAlgorithm.h>
#include "iparallelworker.h"


class iParticleFileLoader;

class vtkFloatArray;
class vtkPointLocator;
class vtkPoints;
class vtkPolyData;


class iParticleDensityEstimator : public vtkAlgorithm, protected iParallelWorker
{

public:

	vtkTypeMacro(iParticleDensityEstimator,vtkAlgorithm);
	static iParticleDensityEstimator* New(iParticleFileLoader *loader = 0);

	void SetNumNeighbors(int num);
	inline int GetNumNeighbors() const { return mNumNeighbors; }

	void ComputeDensity(vtkPoints *points, vtkFloatArray *scalars, int varIn, int varOut);

protected:

	iParticleDensityEstimator(iParticleFileLoader *loader);
	virtual ~iParticleDensityEstimator();

	virtual int ExecuteStep(int step, iParallel::ProcessorInfo &p);
	virtual void PrepareForStep();

	int mNumNeighbors;
	vtkPolyData *mData;
	vtkPointLocator *mLocator;

	//
	//  Work variables
	//
	int wVarIn, wVarOut;
};

#endif  // IPARTICLEDENSITYESTIMATOR_H

