//LICENSE A


#include "iparticleprobefilter.h"


#include "ierror.h"
#include "iviewsubject.h"

#include <vtkPointData.h>
#include <vtkPointLocator.h>

//
//  Templates
//
#include "igenericfilter.tlh"


iParticleProbeFilter::iParticleProbeFilter(iDataConsumer *consumer) : iProbeFilter(consumer)
{
}


void iParticleProbeFilter::ProvideOutput()
{
	vtkDataSet *input = this->InputData();
	vtkDataSet *output = this->OutputData();
	vtkDataSet *source = this->SourceData();

	output->ShallowCopy(input);

	if(input->IsA("vtkPointSet") && output->IsA("vtkPointSet"))
	{
		vtkDataArray *atts = vtkDataSet::SafeDownCast(source)->GetPointData()->GetScalars();
		if(atts!=0 && atts->GetSize()>0)
		{
			vtkFloatArray *outScalars = vtkFloatArray::New(); IERROR_CHECK_MEMORY(outScalars);
			outScalars->SetNumberOfComponents(atts->GetNumberOfComponents());
			
			vtkPointLocator *locator = vtkPointLocator::New(); IERROR_CHECK_MEMORY(locator);
			locator->SetDataSet(vtkDataSet::SafeDownCast(source));
			locator->BuildLocator();

			vtkPoints *pts = vtkPointSet::SafeDownCast(input)->GetPoints();
			vtkIdType i, l, n = pts->GetNumberOfPoints();
			for(i=0; i<n; i++)
			{
				l = locator->FindClosestPoint(pts->GetPoint(i)); 
				if(l != -1) outScalars->InsertNextTuple(atts->GetTuple(l)); else outScalars->InsertNextTuple(atts->GetTuple(0));
			}

			output->GetPointData()->SetScalars(outScalars);
			outScalars->Delete();
			locator->Delete();
		}
	}
}


