//LICENSE A


#ifndef IPARTICLESPLITTER_H
#define IPARTICLESPLITTER_H


#include "igenericfilter.h"
#include <vtkPolyDataAlgorithm.h>


#include "iarray.h"
#include "ipair.h"

class iParticleViewSubject;

class vtkPoints;
class vtkCellArray;
class vtkFloatArray;
class vtkFloatArray;


class iParticleSplitter : public iGenericPolyDataFilter<vtkPolyDataAlgorithm>
{
	
	friend class iParticleViewSubject;
	iGenericFilterTypeMacro(iParticleSplitter,vtkPolyDataAlgorithm);

public:
	
	inline int GetNumberOfPieces() const { return mRanges.Size(); }

	void SetSplitVariable(int v);
	inline int GetSplitVariable() const { return mSplitVariable; }
	
	void SetSorterMode(bool s);
	inline bool GetSorterMode() const { return mSorterMode; }

	const iPair& GetPieceRange(int n) const { return mRanges[n]; }
	void SetPieceRange(int n, const iPair &p);

	bool CreatePiece(const iPair &p);
	bool DeletePiece(int n);

	void TakeOverData(bool);
	float GetMemorySize() const;

protected:
	
	virtual ~iParticleSplitter();

	virtual void ProvideOutput();
	
private:

	int mSplitVariable;
	bool mOwnsData, mSorted, mSorterMode;

	iArray<iPair> mRanges;

	vtkPoints *mSavedPoints;
	vtkCellArray *mSavedVerts;
	vtkFloatArray *mSavedNorms;
	vtkFloatArray *mSavedVars;

	unsigned long mSortingMTime;
};

#endif // IPARTICLESPLITTER_H
 
