//LICENSE A


#ifndef IPARTICLEVIEWSUBJECT_H
#define IPARTICLEVIEWSUBJECT_H


#include "ipaintedviewsubject.h"


#include "ipair.h"


class iEventObserver;
class iParticleConnector;
class iParticleViewSubject;
class iParticleSplitter;
class iViewObject;

class vtkAlgorithmOutput;
class vtkPolyData;


class iParticleViewInstance : public iPaintedViewInstance
{
	
	friend class iParticleViewSubject;

public:
	
	iDataConsumerTypeMacro(iParticleViewInstance,iPaintedViewInstance);
		
	iObjectSlotMacro1SV(Type,int);

	iObjectSlotMacro1SV(ScaleVar,int);
	iObjectSlotMacro1SV(ScaleFactor,float);
	
	iObjectSlotMacro1SV(FixedSize,float);
	iObjectSlotMacro1SV(LineWidth,int);
	iObjectSlotMacro1SV(AutoScaled,bool);

	iObjectSlotMacro2S(ConnectVar,int);
	iObjectSlotMacro2S(ConnectionBreakVar,int);

protected:

	void SetInputConnection(vtkAlgorithmOutput *port);

	//
	//  Inherited members
	//

protected:
	
	iParticleViewInstance(iParticleViewSubject *owner);
	virtual ~iParticleViewInstance();
	virtual void ConfigureBody();
	virtual void ConfigureMainPipeline(iViewSubjectPipeline *p, int id);
	virtual iViewSubjectPipeline* CreatePipeline(int id);

	virtual void ShowBody(bool s);
	virtual bool CanBeShown() const;

	virtual void UpdateColorBars(){}

	virtual bool IsConnectedToPaintingData() const;

private:

	//
	//  VTK members
	//
	iParticleConnector *mConnector;
	iEventObserver *mCameraEventObserver;
	iParticleViewSubject *mRealOwner;
};


class iParticleViewSubject : public iPaintedViewSubject
{

public:
	
	iViewSubjectTypeMacro1(iParticleView,iPaintedViewSubject);
	static iParticleViewSubject* New(iViewObject *parent, const iDataType &type, const iColor &color);

	//
	//  Splitter functionality (must be first on the list)
	//
	iType::ps_int SplitVar;
	iObjectSlotMacro1SV(SplitVar,int);
	iObjectPropertyMacro3V(SplitRanges,Pair);
	iObjectPropertyMacro1S(SplitRangesTiled,Bool);

	//
	//  Group properties
	//
	iType::vsp_int Type;

 	iType::vsp_int ScaleVar;
	iType::vsp_float ScaleFactor;
	
	iType::vsp_float FixedSize;
	iType::vsp_int LineWidth;
	iType::vsp_bool AutoScaled;

	iType::vsp_int ConnectVar;
	iType::vsp_int ConnectionBreakVar;

	iType::ps_int RenderSortVar;
	iObjectSlotMacro1SV(RenderSortVar,int);

	virtual void BecomeClone(iViewSubject *other);

protected:
	
	iParticleViewSubject(iViewObject *parent, const iDataType &type, const iColor &color = iColor::Black());
	virtual ~iParticleViewSubject();

	virtual void InitInstances();

	virtual bool CreateInstance();
	virtual bool RemoveInstance(int n);

	virtual int DepthPeelingRequestBody() const;

	iParticleSplitter *mSplitter;
};

#endif // IPARTICLEVIEWOBJECT_H
 
