//LICENSE A


#ifndef IPICKER_H
#define IPICKER_H


#include "iextendableobject.h"
#include "iviewmodulecomponent.h"


#include "iposition.h"

class iActor;
class iDataFormatter;
class iDataType;

class vtkPicker;
class vtkPolyData;
class vtkSphereSource;


namespace iParameter
{
	namespace PickMethod
	{
		//
		// Picker parameters
		//
		const int Cell =		0;
		const int Point =		1;
		const int Object =		2;
	};
};


class iPicker : public iExtendableObject, public iViewModuleComponent
{

	IPOINTER_AS_USER(DataFormatter);

public:

	vtkTypeMacro(iPicker,iExtendableObject);
	static iPicker* New(iViewModule *vm = 0);

	iObjectPropertyMacro1S(Accuracy,Float);
	iObjectPropertyMacro1S(PickMethod,Int);
	iObjectPropertyMacro1S(PointSize,Float);
	iPropertyQuery<iType::Vector> Position;

	inline const iPosition& GetPosition() const { return mPosition; }
	inline const iVector3D& GetBoxPosition() const { return mPosition.BoxValue(); }
	inline const iString& GetObjectName() const { return mSubjectName; }

	void ShowPickedPoint(bool s);

	inline vtkPicker* GetHandler() const { return mHandler; }

	void UpdateReport();
	virtual void Modified();

protected:

	iPicker(iViewModule *vm, const iString &fname, const iString &sname);
	virtual ~iPicker();

	virtual void UpdateReportBody();

private:

	iPosition mPosition;
	iString mSubjectName;
	const iDataType *mDataTypePointer;
	vtkPolyData *mProbeInput;

	//
	//  Actual workers
	//
	vtkPicker *mHandler;

	//
	//  Graphical representation
	//
	iActor *mPointActor;
	vtkSphereSource *mPointSource;
};


class iPickerExtension : public iObjectExtension
{

	iObjectExtensionDeclareAbstractMacro(iPicker);

public:

	virtual bool IsUsingData(const iDataType &type) const = 0;
	virtual void ModifyActor(const iDataType &type, iActor *actor) = 0;
	virtual void AddInfo(const iDataType &type) = 0;
};

#endif  // IPICKER_H
