//LICENSE A


#ifndef IPIECEWISEFUNCTION_H
#define IPIECEWISEFUNCTION_H


#include "vtkObject.h"


#include "iarray.h"
#include "ipair.h"

class iPiecewiseFunctionHelper;
class iPiecewiseFunctionUser;

class vtkObject;
class vtkPiecewiseFunction;


class iPiecewiseFunction : public vtkObject
{
	
	friend class iPropertyPiecewiseFunction;
	friend class iPiecewiseFunctionHelper;

public:
	
	vtkTypeMacro(iPiecewiseFunction,vtkObject);
	static iPiecewiseFunction* New(float ymin = 0.0f, float ymax = 1.0f);
	
	inline int N() const { return mArray.Size(); }
	inline const iPair& P(int i) const { return mArray[i]; }
	inline float X(int i) const { return mArray[i].X; }
	inline float Y(int i) const { return mArray[i].Y; }
	inline float Min() const { return mYmin; }
	inline float Max() const { return mYmax; }

	void SetMinMax(float ymin, float ymax);
	void AddPoint(int n);
	void AddPoint(const iPair &p);
	void RemovePoint(int n);
	void MovePoint(int n, const iPair &p);
	int FindPoint(const iPair &p, float &d) const;
	float GetValue(float x) const;

	inline void AddPoint(float x, float y){ this->AddPoint(iPair(x,y)); }
	inline void MovePoint(int n, float x, float y){ this->MovePoint(n,iPair(x,y)); }

	void Copy(const iPiecewiseFunction *f);
	void Reset();
	
	vtkPiecewiseFunction* GetVTKFunction() const;

protected:
	
	iPiecewiseFunction(float ymin, float ymax, int inc);
	virtual ~iPiecewiseFunction();

	iOrderedArray<iOrderedPair> mArray;
	float mYmin, mYmax;
	iPiecewiseFunctionHelper *mHelper;
};

#endif // IPIECEWISEFUNCTION_H

