//LICENSE A

//
//  A representation for a single particle
//  

#ifndef IPOINTGLYPH_H
#define IPOINTGLYPH_H


#include <vtkPolyDataAlgorithm.h>


class iDataConsumer;


namespace iParameter
{
	namespace PointGlyphType
	{
		const int Point =			0;
		const int Sphere =			1;
		const int Tetrahydron =		2;
		const int Cube =			3;
		const int Octahedron =		4;
		const int Icosahedron =		5;
		const int Dodecahedron =	6;
		const int Cone =			7;
		const int Cylinder =		8;
		const int Arrow =			9;
		const int Cluster =			10;
		const int Galaxy =			11;
		
		const int SIZE =			12;
	};
};


class iPointGlyph: public vtkPolyDataAlgorithm
{

public:

	vtkTypeMacro(iPointGlyph,vtkPolyDataAlgorithm);
	static iPointGlyph* New();
	
	inline int GetType() const { return mType; }
	void SetType(int m);

	bool GetOriented() const;
	const char* GetName() const;
	
	static bool GetOriented(int t);
	static const char* GetName(int t);

	vtkPolyData* GetData();

protected:
	
	iPointGlyph();
	virtual ~iPointGlyph();
	
	virtual int	RequestData(vtkInformation *request, vtkInformationVector **inputVector, vtkInformationVector *outputVector);

private:

	void CreateData(int res);
	void ScaleData(vtkPolyData *d, double fac);
	void SwapAxes(vtkPolyData *d, int a1, int a2);
	void ShiftData(vtkPolyData *d, int a, double dx);

	int mType;
	vtkPolyData *mData[iParameter::PointGlyphType::SIZE];

	//
	//  Block for better control
	//
	void GetOutput();
	void OutputData();
};

#endif // IPOINTGLYPH_H


