//LICENSE A

//
// A property for an Object with multiple Instances (MarkerObject, ViewSubject)
//

#ifndef IPROPERTYMULTIOBJECT_H
#define IPROPERTYMULTIOBJECT_H


#include "iproperty.h"


template<class Object, class Instance, iType::TypeId id>
class iPropertyMultiObject : public iPropertyDataVariable<id>
{

public:

	typedef typename iType::Traits<id>::Type ArgT;
	typedef bool (Instance::*SetterType)(ArgT val);
	typedef ArgT (Instance::*GetterType)() const;

	iPropertyMultiObject(Object *owner, SetterType setter, GetterType getter, const iString &fname, const iString &sname, int rank = 0);

	virtual int Size() const;
	virtual bool IsFixedSize() const { return false; }

	virtual ArgT GetValue(int i) const;
	virtual bool SetValue(int i, ArgT v) const;

protected:

	virtual bool Resize(int, bool force = false){ return false; }

private:

	Object *mRealOwner;
	SetterType mSetter;
	GetterType mGetter;
};



template<class Object, class Instance, iType::TypeId id>
iPropertyMultiObject<Object,Instance,id>::iPropertyMultiObject(Object *owner, SetterType setter, GetterType getter, const iString &fname, const iString &sname, int rank) : iPropertyDataVariable<id>(owner,fname,sname,rank)
{
	this->mRealOwner = owner;
	this->mSetter = setter;
	this->mGetter = getter;
}


template<class Object, class Instance, iType::TypeId id>
int iPropertyMultiObject<Object,Instance,id>::Size() const
{
	return this->mRealOwner->mInstances.Size();
}


template<class Object, class Instance, iType::TypeId id>
typename iPropertyMultiObject<Object,Instance,id>::ArgT iPropertyMultiObject<Object,Instance,id>::GetValue(int i) const
{
	return (this->mRealOwner->mInstances[i]->*this->mGetter)();
}


template<class Object, class Instance, iType::TypeId id>
bool iPropertyMultiObject<Object,Instance,id>::SetValue(int i, ArgT v) const
{
	return (this->mRealOwner->mInstances[i]->*this->mSetter)(v);
}

#endif // IPROPERTYMULTIOBJECT_H

