//LICENSE A


#include "ireplicatedelement.h"


#include "ierror.h"
#include "iviewsubject.h"


iReplicatedElement::iReplicatedElement(bool isData) : mIsData(isData)
{
	int i;
	for(i=0; i<6; i++) mReplicationFactors[i] = mOldReplicationFactors[i] = 0;

	mParent = 0;
}


iReplicatedElement::~iReplicatedElement()
{
	if(mParent != 0)
	{
		mParent->UnRegisterReplicatedElement(this); 
	}
}


void iReplicatedElement::ReplicateAs(iViewSubject *parent)
{
	if(mParent != 0)
	{
		mParent->UnRegisterReplicatedElement(this); 
	}

	mParent = parent;

	if(mParent != 0)
	{
		mParent->RegisterReplicatedElement(this,mIsData);
	}

	this->UpdateReplicas();
}


void iReplicatedElement::UpdateReplicas(bool force)
{
	int i;
	bool work = force;

	this->UpdateReplicasHead();

	if(mParent != 0)
	{
		if(mIsData == mParent->IsOptimizedForQuality())
		{
			for(i=0; i<6; i++) mReplicationFactors[i] = mParent->mReplicationFactors[i];
		}
		else
		{
			for(i=0; i<6; i++) mReplicationFactors[i] = 0;
		}
	}

	for(i=0; !work && i<6; i++)
	{
		if(mOldReplicationFactors[i] != mReplicationFactors[i]) work = true;
	}

	if(work)
	{
		this->UpdateReplicasBody();
		for(i=0; i<6; i++) mOldReplicationFactors[i] = mReplicationFactors[i];
	}
}


void iReplicatedElement::SetReplicationFactors(const int n[6])
{
	int i;
	for(i=0; i<6; i++) mReplicationFactors[i] = n[i];
	this->UpdateReplicas();
}


void iReplicatedElement::GetReplicationFactors(int n[6]) const
{
	int i;
	for(i=0; i<6; i++) n[i] = mReplicationFactors[i];
}
