//LICENSE A


//
//  A volume that can be replicated periodically. This is a preferred way to replicate, since
//  we create no new data and re-use mappers.
//
#ifndef IREPLICATEDVOLUME_H
#define IREPLICATEDVOLUME_H


#include "igenericprop.h"
#include <vtkVolume.h>
#include "ireplicatedelement.h"
#include "iviewmodulecomponent.h"


#include "iarray.h"

class vtkFrustumCoverageCuller;


class iReplicatedVolume : public iGenericProp<vtkVolume>, public iReplicatedElement, public iViewModuleComponent
{
	
public:
	
	vtkTypeMacro(iReplicatedVolume,vtkVolume);
	static iReplicatedVolume* New(iViewSubject *owner = 0);

	virtual double* GetBounds();

	inline vtkTransform* GetTransform() const { return this->Transform; }

protected:
	
	iReplicatedVolume(iViewSubject *owner);
	virtual ~iReplicatedVolume();
	
	virtual void UpdateGeometry(vtkViewport *vp);

	virtual void UpdateReplicasBody();

private:

	void CreateReplica(int i, int j, int k);

	struct Replica
	{
		vtkVolume *Volume;
		int Pos[3];
		Replica(){ Volume = 0; Pos[0] = Pos[1] = Pos[2] = 0; }
	};

	iArray<Replica> mReplicas;
	vtkFrustumCoverageCuller *mCuller;
};

#endif // IREPLICATEDVOLUME_H
