//LICENSE A

//
//  This is base class for all parallellizable pipelines
//

#ifndef ISERIALPIPELINE_H
#define ISERIALPIPELINE_H


#include "igenericfilter.h"


template<class Filter, class InputType>
class iSerialPipeline : public Filter
{

public:

	virtual iSerialPipeline<Filter,InputType>* Copy() const = 0;

	virtual float GetMemorySize()
	{
		return this->Filter::GetMemorySize() + this->GetContentsMemorySize();
	}

	virtual void UpdateContents(int n, int info = 0) = 0;

	void SetGlobalInput(InputType *i)
	{
		this->mGlobalInput = i;
	}

	InputType* GetGlobalInput() const {	return this->mGlobalInput; }

	virtual void SetNthInput(int num, vtkDataObject *input){ Filter::SetNthInput(num,input); } // vtkProcessObject::SetNthInput(num,input) is protected

protected:

	iSerialPipeline(iViewSubject *vo, int numInputs) : Filter(vo,numInputs,true,true)
	{
		this->mGlobalInput = 0;
	}

	virtual float GetContentsMemorySize() const = 0;

	InputType *mGlobalInput;
};


//
//  Create specific filters
//
typedef iSerialPipeline<iAbstractPolyDataToPolyDataFilter,vtkPolyData>  iPolyDataToPolyDataPipeline;
typedef iSerialPipeline<iAbstractGridDataToPolyDataFilter,vtkImageData> iGridDataToPolyDataPipeline;
typedef iSerialPipeline<iAbstractGridDataToGridDataFilter,vtkImageData> iGridDataToGridDataPipeline;
typedef iSerialPipeline<iAbstractAnyDataToPolyDataFilter,vtkDataSet> iAnyDataToPolyDataPipeline;

#endif  // ISERIALPIPELINE_H
