//LICENSE A

//
//  EventObservers used by Shell
//

#ifndef ISHELLEVENTOBSERVERS_H
#define ISHELLEVENTOBSERVERS_H


#include "ieventobserver.h"
#include "ishellcomponent.h"


#include "iarray.h"
#include "istring.h"


class iExecutionEventObserver : public iEventObserver, public iShellComponent
{

public:
	
	vtkTypeMacro(iExecutionEventObserver,iEventObserver);

	//
	//  Public for manual driving
	//
	void Start();
	void Finish();
	void SetProgress(double fraction);
	void SetLabel(const iString &label);

	void SetInterval(double start, double length);

	void PostError(const iString &text);
	void Reset();

	const iString& LastErrorMessage() const;

	inline bool IsAborted() const { return mAborted; }
	inline bool IsInError() const { return mInError; }

protected:

	iExecutionEventObserver(iShell *s);

	virtual void ExecuteBody(vtkObject *caller, unsigned long event, void *data);

	virtual void OnStart() = 0;
	virtual void OnFinish() = 0;
	virtual void OnProgress(double fraction, bool &abort) = 0;
	virtual void OnSetLabel(const iString &label) = 0;

	static iExecutionEventObserver* New(); // not implemented

private:

	struct Item
	{
		double Start, Length, PrevValue;
		iString ErrorMessages;
		Item();
	};

	iArray<Item> mStack;
	bool mAborted, mInError;

#ifdef I_DEBUG
	double prev;
	double time[10];
	iString label;
#endif
};


class iParallelUpdateEventObserver : public iEventObserver, public iShellComponent
{
	
public:
	
	vtkTypeMacro(iParallelUpdateEventObserver,iEventObserver);

	virtual void UpdateInformation() = 0;

protected:

	iParallelUpdateEventObserver(iShell *s);

	virtual void ExecuteBody(vtkObject *caller, unsigned long event, void *data);

	static iParallelUpdateEventObserver* New(); // not implemented (created directly bu shell)
};

#endif // ISHELLEVENTOBSERVERS_H

