//LICENSE A


#include "ishiftpolydatafilter.h"


#include "ierror.h"

#include <vtkFloatArray.h>
#include <vtkPointData.h>
#include <vtkPolyData.h>


//
//  Templates
//
#include "igenericfilter.tlh"


iShiftPolyDataFilter::iShiftPolyDataFilter(iDataConsumer *consumer) : iGenericPolyDataFilter<vtkPolyDataAlgorithm>(consumer,1,true)
{
	mShift = 0.003f;
}


void iShiftPolyDataFilter::SetShift(float s)
{
	mShift = s;
	this->Modified();
}


void iShiftPolyDataFilter::ProvideOutput()
{
	vtkPolyData *input = this->InputData();
	vtkPolyData *output = this->OutputData();

	output->DeepCopy(input);

	if(fabs(mShift) < 1.0e-30) return;

	vtkFloatArray *nor = vtkFloatArray::SafeDownCast(output->GetPointData()->GetNormals());
	vtkPoints *poi = output->GetPoints();

	vtkIdType n = nor->GetNumberOfTuples();
	if(n != poi->GetNumberOfPoints()) return;

	n *= 3;
	vtkIdType l; 
	float *nptr = (float *)nor->GetVoidPointer(0);

	switch(poi->GetDataType())
	{
	case VTK_FLOAT:
		{
			float *xptr = (float *)poi->GetVoidPointer(0);
			for(l=0; l<n; l++) xptr[l] += mShift*nptr[l];
			return;
		}
	case VTK_DOUBLE:
		{
			double *xptr = (double *)poi->GetVoidPointer(0);
			for(l=0; l<n; l++) xptr[l] += mShift*nptr[l];
			return;
		}
	default:
		{
			vtkErrorMacro("Invalid points data type");
			return;
		}
	}
}

