//LICENSE A

//
//  A stereo (two-eyed) image
//
#ifndef ISTEREOIMAGE_H
#define ISTEREOIMAGE_H


#include "iimage.h"


class iStereoImage
{

public:

	iStereoImage();
	iStereoImage(const iImage &img);
	iStereoImage(const iStereoImage &img);

	const iImage& Eye(int n) const;
	const iImage& LeftEye() const;
	const iImage& RightEye() const;

    iStereoImage& operator=(const iStereoImage &img);   // impl-shared copy
    iStereoImage& operator=(const iImage &img);   // impl-shared copy
	void CopyLeftEyeToRightEye();
	void SetStereo(bool s);

	//
	//  iImage functions
	//
	void Scale(int w, int h, iImage::ScaleMode m = iImage::_Both, iImage::ScaleFilter f = iImage::_None);
//	void Smooth(float dx, float dy);
//	void Clear();
	void ReplaceData(int n, vtkImageData *data);
	void Blend(const iImage& im, float op);
	void Blend(const iStereoImage& im, float op);
	void Overlay(int x, int y, const iImage& ovr, float opacity = 1.0, bool masking = true);
	void Overlay(int x, int y, const iStereoImage& ovr, float opacity = 1.0, bool masking = true);
	bool CombineInPseudoColor(const iStereoImage &imRed, const iStereoImage &imGreen, const iStereoImage &imBlue);
	void DrawFrame(int width, const iColor &color);

	inline bool IsStereo() const { return mIsStereo; }
	inline bool IsEmpty() const { return mEyeImage[0].IsEmpty(); }
	inline int Depth() const { return mEyeImage[0].Depth(); }
	inline int Width() const { return mEyeImage[0].Width(); }
	inline int Height() const { return mEyeImage[0].Height(); }

	void SetTag(const iString& tag);
	inline const iString& GetTag() const { return mTag; }

private:

	iImage mEyeImage[2];
	bool mIsStereo;
	iString mTag;
};


inline const iImage& iStereoImage::Eye(int n) const
{ 
	return mEyeImage[(n==0 || !mIsStereo)?0:1]; 
}


inline const iImage& iStereoImage::LeftEye() const
{
	return mEyeImage[0];
}


inline const iImage& iStereoImage::RightEye() const
{
	return mEyeImage[mIsStereo?1:0];
}

#endif // ISTEREOIMAGE_H

