//LICENSE A


//
//	A replacement for vtkStreamLineFilter which is VERY SLOW and takes all of the memory
//

#ifndef ISTREAMLINEFILTER_H
#define ISTREAMLINEFILTER_H


#include <vtkStreamTracer.h>
#include "igenericfilter.h"


class iDataLimits;

class vtkImageData;
class vtkPointSet;


namespace iParameter
{
	namespace StreamLine
	{
		namespace Direction
		{
			const int UpStream = 0;
			const int DownStream = 1;
			const int Forward = 2;
			const int Backward = 3;
			const int BothWays = 4;

			inline bool IsValid(int m){ return m>=0 && m<=4; }
			//const int COUNT = 5;
		};
	};
};


class iStreamLineFilter : public iGenericFilter<vtkStreamTracer,vtkDataSet,vtkPolyData>
{

	iGenericFilterTypeMacro(iStreamLineFilter,vtkStreamTracer);

public:

	vtkDataSet* SourceData();

	void SetLength(float d);
	void SetQuality(int q);
	void SetDirection(int d);
	void SetSplitLines(bool s);
	void SetMinimumSpeed(float v);

	inline float GetLength() const { return mLength; }
	inline int GetQuality() const { return mQuality; }
	inline int GetDirection() const { return mDir; }
	inline bool GetSplitLines() const { return mSplitLines; }
	inline float GetMinimumSpeed() const { return mVmin; }

protected:
	
	virtual void ProvideOutput();

	virtual vtkIdType GetVector(double x[3], float v[3]);
	virtual int FollowLine(double x[3], double &d, float &h, float eps, vtkIdType l);
	
	virtual bool DefinePointers(vtkImageData *input);
	virtual void AssignScalars(float *p, vtkIdType l);
	virtual void GetDmin(vtkIdType l, float &ddmin, float &dx);

	static vtkIdType GetVectorWithCIC(double x[3], float v[3], float *ptrdata, int dims[3], double origin[3], double spacing[3], bool per[3]);

	//
	//  Work variables
	//
	const iDataLimits *wLimits;
	int wDims[3], wNsca;
	float *wPtrVec, *wPtrSca;
	double wOrg[3], wSpa[3];
	bool wPer[3];

	float mDmin;

private:

	void ProvideOutputForImageData(vtkImageData *input, vtkPolyData *output, vtkPointSet *source);

	float mVmin, mEps;
	float mLength;
	int mDir, mQuality, mItMax, mAxisDir;
	bool mSplitLines;

	/* blocked */ vtkDataSet *GetSource(){ return this->vtkStreamTracer::GetSource(); }
};

#endif  // ISTREAMLINEFILTER_H


