//LICENSE A


#ifndef ISTRETCH_H
#define ISTRETCH_H


#include "imath.h"

class iString;


namespace iStretch
{
	const int Lin = 0;
	const int Log = 1;
	const int Count = 2;

	template<class T> T Apply(T f, int id, bool up);
	template<class T> T Reset(T f, int id);

	const iString& GetName(int s);
	int GetId(const iString& name);
};


//
//  stretch and restore the data
//
namespace iStretch
{
	template<class T>
	inline T Apply(T f, int id, bool up)
	{
		const T zero = (T)0.0;
		const T tiny = (T)1.0e-36;
		const T invalid = (T)36.0;

		switch(id)
		{
		case Log:
			{
				if(f > zero) 
				{
					return iMath::Log10(tiny+f);
				}
				else 
				{
					return (up ? invalid : -invalid);
				}
				break;
			}
		default: return f;
		}
	}


	template<class T>
	inline T Reset(T f, int id)
	{
		switch(id)
		{
		case Log: 
			{
				return iMath::Pow10(f);
				break;
			}
		default: return f;
		}
	}
};

#endif // ISTRETCH_H

