//LICENSE A


#ifndef ISURFACEVIEWSUBJECT_H
#define ISURFACEVIEWSUBJECT_H


#include "imultiviewsubject.h"


class iSurfaceViewSubject;
class iViewObject;


namespace iParameter
{
	namespace SurfaceMethod
	{
		//
		// SurfaceViewSubject parameters
		//
		const int Isosurface = 0;
		const int Sphere = 1;
		const int Plane = 2;
	};
};


class iSurfaceViewInstance : public iMultiViewInstance, public iPropPlacementHelper
{
	
	friend class iSurfaceViewSubject;

public:
	
	iDataConsumerTypeMacro(iSurfaceViewInstance,iMultiViewInstance);
	
	iObjectSlotMacro1SV(Method,int);
	iObjectSlotMacro1SV(NormalsFlipped,bool);
	iObjectSlotMacro1SV(IsoVar,int);
	iObjectSlotMacro1SV(IsoLevel,float);
	iObjectSlotMacro1SV(IsoMethod,int);
	iObjectSlotMacro1SV(IsoSmoothing,int);
	iObjectSlotMacro1SV(IsoReduction,int);
	iObjectSlotMacro1SV(IsoOptimization,bool);
	iObjectSlotMacro1SV(AlternativeReductionMethod,bool);

	//
	//  Slot wrappers
	//
	bool SetPlaneDirection(const iVector3D& v){ return this->iPropPlacementHelper::SetDirection(v); }
	const iVector3D& GetPlaneDirection() const { return this->iPropPlacementHelper::GetDirection(); }

	bool SetObjectPosition(const iVector3D& v){ return this->iPropPlacementHelper::SetBoxPosition(v); }
	const iVector3D& GetObjectPosition() const { return this->iPropPlacementHelper::GetBoxPosition(); }

	bool SetSphereSize(double p){ return this->iPropPlacementHelper::SetBoxSize(p); }
	double GetSphereSize() const { return this->iPropPlacementHelper::GetBoxSize(); }

protected:
	
	iSurfaceViewInstance(iSurfaceViewSubject *owner);
	virtual ~iSurfaceViewInstance();
	virtual void ConfigureBody();
	virtual void ConfigureMainPipeline(iViewSubjectPipeline *p, int id);

	virtual void ShowBody(bool s);
	virtual bool CanBeShown() const;

	virtual iViewSubjectPipeline* CreatePipeline(int id);

	virtual void UpdateDirection();
	virtual void UpdatePosition();
	virtual void UpdateSize();

	void OutputProperties(iString& str) const;
};


class iSurfaceViewSubject : public iMultiViewSubject
{
	
	friend class iSurfaceViewInstance;

public:
	
	iViewSubjectTypeMacro(iSurfaceView,iMultiViewSubject);
	
	iType::vsp_int Method;
	iType::vsp_bool NormalsFlipped;
	iType::vsp_int IsoVar;
	iType::vsp_float IsoLevel;
	iType::vsp_int IsoMethod;
	iType::vsp_int IsoSmoothing;
	iType::vsp_int IsoReduction;
	iType::vsp_bool IsoOptimization;
	iType::vsp_bool AlternativeReductionMethod;
	iType::vsp_vector PlaneDirection;
	iType::vsp_double SphereSize;
	iType::vsp_vector Position;

	void OutputProperties(int i, iString& str) const;

	bool InterpolateBetweenTwoInstances(int i1, int i2, int n, bool testOnly = false);

protected:
	
	iSurfaceViewSubject(iViewObject *parent, const iDataType &type);
	virtual ~iSurfaceViewSubject();

	virtual int DepthPeelingRequestBody() const;
};

#endif // ISURFACEVIEWSUBJECT_H

