//LICENSE A

//
//  Some useful system-level functions
//
#ifdef _WIN32
#include <windows.h>
//#include <winbase.h>
#else
#include <limits.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <unistd.h>
#endif


#include <vtkTimerLog.h>


namespace iSystem
{
	//
	//  Sleeps for a given # of milliseconds
	//
	void Sleep(int msec)
	{
		if(msec <= 0) return;
#ifdef _WIN32
		::Sleep(msec);
#else
		usleep((unsigned long)msec*1000UL);
#endif
	}


	//
	//  Gives the physical memory for the machine
	//
#define __MB	(float)1048576.0
	float GetMemoryLimit()
	{
#ifdef _WIN32
		MEMORYSTATUS r;
		::GlobalMemoryStatus(&r);
		return float(r.dwTotalPhys)/__MB;
#else
		static struct rlimit r;
		int d = getrlimit(RLIMIT_DATA,&r);
		float v1 = float(r.rlim_cur)/__MB;
#if defined(_SC_PHYS_PAGES) && defined(_SC_PAGESIZE)
		unsigned long np = sysconf(_SC_PHYS_PAGES);
		unsigned long ps = sysconf(_SC_PAGESIZE);
		float v2 = float(np)*float(ps)/__MB;
		if(v1 > v2) v1 = v2;
#endif
		if(d == 0) return v1; else return ULONG_MAX/__MB;
#endif
	}


	//
	//  Get the endiness of the machine.
	//
	bool IsBigEndianMachine()
	{
		unsigned char c[4];
		c[0] = c[1] = c[2] = (unsigned char)0;
		c[3] = (unsigned char)1;
		
		return (*((int *)c) == 1);
	}
};
