//LICENSE A


#include "itensorfieldglyphpipeline.h"


#include "idatalimits.h"
#include "ierror.h"
#include "ipointglyph.h"
#include "ireplicatedpolydata.h"
#include "iresampleimagedatafilter.h"
#include "itensorglyphfilter.h"
#include "itensorfieldviewsubject.h"

//
//  Templates (needed for some compilers)
//
#include "iarray.tlh"
#include "igenericfilter.tlh"
#include "iviewsubjectpipeline.tlh"


iPipelineKeyDefineMacro(iTensorFieldGlyphPipeline,Scaling);
iPipelineKeyDefineMacro(iTensorFieldGlyphPipeline,Painting);
iPipelineKeyDefineMacro(iTensorFieldGlyphPipeline,GlyphType);


//
//  iTensorFieldGlyphPipeline class
//
iTensorFieldGlyphPipeline::iTensorFieldGlyphPipeline(iTensorFieldViewInstance *owner) : iFieldGlyphPipeline(owner)
{
	mOwner = owner;

	//
	//  Do VTK stuff
	//	
	mGlyph = iPointGlyph::New(); IERROR_CHECK_MEMORY(mGlyph);

	mGlyphFilter = this->CreateFilter<iTensorGlyphFilter>();
	mGlyph->SetType(iParameter::PointGlyphType::Cube);

	mGlyphFilter->SetInputConnection(mResampleFilter->GetOutputPort());
	mGlyphFilter->SetGlyphConnection(mGlyph->GetOutputPort());

	this->CompletePipeline(mGlyphFilter->GetOutputPort());
	this->UpdateContents();
}


iTensorFieldGlyphPipeline::~iTensorFieldGlyphPipeline()
{
	mGlyph->Delete();
}


void iTensorFieldGlyphPipeline::SetGlyphFilterInput(vtkDataSet *input)
{
	if(mGlyphFilter->InputData() != input)
	{
		mGlyphFilter->SetInputData(input);
	}
}


void iTensorFieldGlyphPipeline::SetGlyphFilterScale(double s)
{
	mGlyphFilter->SetScaleFactor(s);
}


void iTensorFieldGlyphPipeline::UpdateGlyphType()
{
	switch(mOwner->GetGlyphType())
	{
	case 1:
	{
		mGlyph->SetType(iParameter::PointGlyphType::Sphere);
		mGlyphFilter->Modified();
		break;
	}
	default:
	{
		mGlyph->SetType(iParameter::PointGlyphType::Cube);
		mGlyphFilter->Modified();
	}
	};
	this->Modified();
}


void iTensorFieldGlyphPipeline::UpdateScaling()
{
	mGlyphFilter->SetScaling(mOwner->GetScaling());
	this->Modified();
}


void iTensorFieldGlyphPipeline::UpdatePainting()
{
	mGlyphFilter->SetCurrentVar(mOwner->GetPaintVar());
	this->Modified();
}

