//LICENSE A


#include "itensorglyphfilter.h"


#include "ierror.h"

#include <vtkDataSet.h>
#include <vtkPointData.h>
#include <vtkTensorGlyph.h>

//
//  Templates
//
#include "igenericfilter.tlh"


iTensorGlyphFilter::iTensorGlyphFilter(iDataConsumer *consumer) : iGenericFilter<vtkDataSetAlgorithm,vtkDataSet,vtkPolyData>(consumer,1,true)
{
	mWorker = vtkTensorGlyph::New(); IERROR_CHECK_MEMORY(mWorker);
	mWorker->ScalingOn();
	mWorker->ColorGlyphsOn();
	mWorker->ClampScalingOn();
	mWorker->SetColorModeToScalars();
}


iTensorGlyphFilter::~iTensorGlyphFilter()
{
	mWorker->Delete();
}


void iTensorGlyphFilter::SetScaling(bool s)
{
	if(s != this->GetScaling())
	{
		mWorker->SetScaling(s?1:0);
		this->Modified();
	}
}


bool iTensorGlyphFilter::GetScaling() const
{
	return (mWorker->GetScaling() != 0);
}


void iTensorGlyphFilter::SetScaleFactor(float s)
{
	if(s != this->GetScaleFactor())
	{
		mWorker->SetScaleFactor(s);
		this->Modified();
	}
}


float iTensorGlyphFilter::GetScaleFactor() const
{
	return mWorker->GetScaleFactor();
}


void iTensorGlyphFilter::SetCurrentVar(int n)
{ 
	if(n>=0 && n!= mCurVar)
	{
		mCurVar = n;
		this->Modified();
	}
}


void iTensorGlyphFilter::ProvideOutput()
{
	vtkDataSet *input = this->InputData();
	
	vtkDataSet *workerInput = input->NewInstance(); IERROR_CHECK_MEMORY(workerInput);
	workerInput->ShallowCopy(input);

	if(this->InputData()->GetPointData()!=0 && this->InputData()->GetPointData()->GetScalars()!=0)
	{
		//
		//  We have scalars
		//
		vtkFloatArray *scalars = iRequiredCast<vtkFloatArray>(INFO,input->GetPointData()->GetScalars());

		int var = mCurVar;
		if(var >= scalars->GetNumberOfComponents())
		{
			var = scalars->GetNumberOfComponents() - 1;
		}

		if(var >= 0) // we color by scalars
		{
			//
			//  vtkTensorGlyph always uses the first scalar component. We simple shift the data by mCurVar values
			//  so that the current value is always the 0 component.
			//
			vtkFloatArray *shiftedScalars = vtkFloatArray::New(); IERROR_CHECK_MEMORY(shiftedScalars);
			shiftedScalars->SetNumberOfComponents(scalars->GetNumberOfComponents());
			shiftedScalars->SetArray(scalars->GetPointer(var),scalars->GetSize(),1);
			workerInput->GetPointData()->SetScalars(shiftedScalars);
			shiftedScalars->Delete();
		}
	}

	mWorker->SetInputData(workerInput);
	mWorker->Update();
	workerInput->Delete();

	this->OutputData()->ShallowCopy(mWorker->GetOutput());
}


void iTensorGlyphFilter::SetGlyphConnection(vtkAlgorithmOutput *port)
{
	mWorker->SetSourceConnection(port);
}

