//LICENSE A


#ifndef ITYPE_H
#define ITYPE_H


#include "icolor.h"
#include "idataid.h"
#include "ipair.h"
#include "istring.h"
#include "ivector3D.h"


//
//  Types of object properties 
//
namespace iType
{
	enum TypeId
	{
		None = 0,
		Int,
		Bool,
		Float,
		Double,
		String,
		DataType,
		Pair,
		Color,
		Vector
	};


	template<TypeId id> struct Traits
	{
	};


	//
	//  "Primitive" types
	//
	template<> struct Traits<Int>
	{
		typedef int Type;
		typedef int ContainerType;
		static const iString& TypeName();
		static iString ValueToString(int val, bool exact);
		static bool StringToValue(const iString &str, int &val);
	};


	template<> struct Traits<Bool>
	{
		typedef bool Type;
		typedef bool ContainerType;
		static const iString& TypeName();
		static iString ValueToString(bool val, bool exact);
		static bool StringToValue(const iString &str, bool &val);
	};


	template<> struct Traits<Float>
	{
		typedef float Type;
		typedef float ContainerType;
		static const iString& TypeName();
		static iString ValueToString(float val, bool exact);
		static bool StringToValue(const iString &str, float &val);
	};


	template<> struct Traits<Double>
	{
		typedef double Type;
		typedef double ContainerType;
		static const iString& TypeName();
		static iString ValueToString(double val, bool exact);
		static bool StringToValue(const iString &str, double &val);
	};


	template<> struct Traits<String>
	{
		typedef const iString& Type;
		typedef iString ContainerType;
		static const iString& TypeName();
		static iString ValueToString(const iString& val, bool exact);
		static bool StringToValue(const iString &str, iString &val);
	};


	template<> struct Traits<DataType>
	{
		typedef const iDataType& Type;
		typedef iDataId ContainerType;
		static const iString& TypeName();
		static const iString& ValueToString(const iDataType& val, bool exact);
		static bool StringToValue(const iString &str, iDataId &val);
	};


	//
	//  Compound types
	//
	template<> struct Traits<Pair>
	{
		typedef const iPair& Type;
		typedef iPair ContainerType;
		static const iString& TypeName();
		static iString ValueToString(const iPair& val, bool exact);
		static bool StringToValue(const iString &str, iPair &val);
		static const int NumFields = 2;
		static const TypeId FieldTypeId = Float;
	};


	template<> struct Traits<Color>
	{
		typedef const iColor& Type;
		typedef iColor ContainerType;
		static const iString& TypeName();
		static iString ValueToString(const iColor& val, bool exact);
		static bool StringToValue(const iString &str, iColor &val);
		static const int NumFields = 4;
		static const TypeId FieldTypeId = Int;
	};


	template<> struct Traits<Vector>
	{
		typedef const iVector3D& Type;
		typedef iVector3D ContainerType;
		static const iString& TypeName();
		static iString ValueToString(const iVector3D& val, bool exact);
		static bool StringToValue(const iString &str, iVector3D &val);
		static const int NumFields = 3;
		static const TypeId FieldTypeId = Double;
	};
};


#endif  // ITYPE_H
