//LICENSE A


#ifndef IUNIFORMGRIDDATA_H
#define IUNIFORMGRIDDATA_H


#include <vtkImageData.h>


class iUniformGridFileLoader;


namespace iParameter
{
	namespace VoxelLocation
	{
		const int Vertex = 0;
		const int Center = 1;

		inline bool IsValid(int m){ return m>=0 && m<=1; }
	};
};


class iUniformGridData : public vtkImageData
{

	friend class iUniformGridFileLoader;

public:
	
	vtkTypeMacro(iUniformGridData,vtkImageData);
	static iUniformGridData* New();

	//
	//  VTK methods
	//
	virtual void Initialize();
	virtual void ShallowCopy(vtkDataObject *d);
	virtual void DeepCopy(vtkDataObject *d);

	//
	//  Special methods
	//
	inline int GetVoxelLocation() const { return mVoxelLocation; }
	void GetPeriodicOffsets(int perOff[3]) const;

	void GetNumCells(int numCells[3]);
	void GetBounds(double min[3], double max[3]);
	void GetGlobalOrigin(double org[3]);

	//
	//  Just keep them here because they should be somewhere
	//
	static void CIC(bool periodic[3], int dims[3], double org[3], double spa[3],  float pos[3], int ijk1[3], int ijk2[3], double d1[3], double d2[3]);
	static void CIC(bool periodic[3], int dims[3], double org[3], double spa[3], double pos[3], int ijk1[3], int ijk2[3], double d1[3], double d2[3]);

protected:
	
	iUniformGridData();
	virtual ~iUniformGridData();

private:

	void SetInternalProperties(int voxLoc, int fileDims[3], int dataDims[3]);

	int mVoxelLocation;
	int mPeriodicOffsets[3];
};

#endif // IUNIFORMGRIDDATA_H

