//LICENSE A


#ifndef IVECTORFIELDVIEWSUBJECT_H
#define IVECTORFIELDVIEWSUBJECT_H


#include "ifieldviewsubject.h"


#include "iposition.h"

class iBoundedDiskSource;
class iBoundedPlaneSource;
class iBoundedSphereSource;
class iMarker;
class iVectorFieldViewSubject;
class iViewObject;

class vtkCellArray;
class vtkFloatArray;
class vtkPoints;


namespace iParameter
{
	namespace VectorField
	{
		namespace Method
		{
			const int Glyph =		0;
			const int StreamLine =	1;
			const int StreamTube =	2;
			const int StreamBand =	3;

			inline bool IsValid(int m){ return m>=0 && m<=3; }
		};
	};
	namespace StreamLine
	{
		namespace Source
		{
			const int Disk =	0;
			const int Plane =	1;
			const int Sphere =	2;
			const int Marker =	3;

			inline bool IsValid(int m){ return m>=0 && m<=3; }
			//const int COUNT =	4;
		};
	};
};


class iVectorFieldViewInstance : public iFieldViewInstance, public iPropPlacementHelper
{
	
	friend class iVectorFieldViewSubject;

public:
	
	iDataConsumerTypeMacro(iVectorFieldViewInstance,iFieldViewInstance);
		
	iObjectSlotMacro1SV(Method,int);
	iObjectSlotMacro1SV(LineWidth,int);
	iObjectSlotMacro1SV(LineQuality,int);
	iObjectSlotMacro1SV(LineLength,float);
	iObjectSlotMacro1SV(LineDirection,int);
	iObjectSlotMacro1SV(TubeSize,int);
	iObjectSlotMacro1SV(TubeRangeFactor,float);
	iObjectSlotMacro1SV(TubeVariationFactor,float);
	iObjectSlotMacro1SV(NumberOfStreamLines,int);
	iObjectSlotMacro1SV(SourceType,int);
	iObjectSlotMacro1SV(SourceOpacity,float);  
	iObjectSlotMacro1SV(ShowSourceObject,bool); 
	iObjectSlotMacro1SV(GlyphBaseSize,int); 

	virtual bool CanBeShown() const;

	virtual void UpdateOnMarkerChange();

protected:
	
	iVectorFieldViewInstance(iVectorFieldViewSubject *owner);
	virtual ~iVectorFieldViewInstance();
	virtual void ConfigureBody();
	virtual void ConfigureMainPipeline(iViewSubjectPipeline *p, int id);

	virtual iViewSubjectPipeline* CreatePipeline(int id);
	virtual void ResetPipelineInput(vtkDataSet *input);
	virtual void ResetBody();

	virtual float GetMemorySize();
	virtual void RemoveInternalData();

	virtual void ShowBody(bool s);

	virtual void UpdateGlyphSize();
	virtual void UpdateGlyphSampleRate();
	virtual void UpdateStreamLineSource();
	virtual void UpdatePipelines();

	virtual void UpdateDirection();
	virtual void UpdatePosition();
	virtual void UpdateSize();

	//
	//  VTK stuff
	//
	vtkPoints *mStreamLineSourcePoints;
	vtkCellArray *mStreamLineSourceVerts;
	vtkFloatArray *mStreamLineSourceNorms;

	iBoundedDiskSource *mSourceDisk;
	iBoundedPlaneSource *mSourcePlane;
	iBoundedSphereSource *mSourceSphere;

private:
	//
	//  Slot wrappers
	//
	bool SetSourceDirection(const iVector3D& v){ return this->iPropPlacementHelper::SetDirection(v); }
	const iVector3D& GetSourceDirection() const { return this->iPropPlacementHelper::GetDirection(); }

	bool SetSourcePosition(const iVector3D& v){ return this->iPropPlacementHelper::SetBoxPosition(v); }
	const iVector3D& GetSourcePosition() const { return this->iPropPlacementHelper::GetBoxPosition(); }

	bool SetSourceSize(double p){ return this->iPropPlacementHelper::SetBoxSize(p); }
	double GetSourceSize() const { return this->iPropPlacementHelper::GetBoxSize(); }
};


class iVectorFieldViewSubject : public iFieldViewSubject
{
	
public:
	
	iViewSubjectTypeMacro2(iVectorFieldView,iFieldViewSubject);
		
	iType::vsp_int Method;
	iType::vsp_int LineWidth;
	iType::vsp_int LineQuality;
	iType::vsp_float LineLength;
	iType::vsp_int LineDirection;
	iType::vsp_int TubeSize;
	iType::vsp_float TubeRangeFactor;
	iType::vsp_float TubeVariationFactor;
	iType::vsp_int NumberOfStreamLines;
	iType::vsp_int SourceType;
	iType::vsp_float SourceOpacity;  
	iType::vsp_vector SourceDirection; 
	iType::vsp_vector SourcePosition; 
	iType::vsp_double SourceSize;
	iType::vsp_bool ShowSourceObject; 
	iType::vsp_int GlyphBaseSize;

protected:
	
	iVectorFieldViewSubject(iViewObject *parent, const iDataType &type, const iDataType &stype);
	virtual ~iVectorFieldViewSubject();
};

#endif // IVECTORFIELDVIEWSUBJECT_H

