//LICENSE A

//
//  EventObservers used by ViewModule and its components
//

#ifndef IVIEWEVENTOBSERVERS_H
#define IVIEWEVENTOBSERVERS_H


#include "ieventobserver.h"
#include "iviewmodulecomponent.h"


class iString;
class iViewModule;

class vtkTimerLog;


class iRenderEventObserver : public iEventObserver, public iViewModuleComponent
{
	
public:
	
	vtkTypeMacro(iRenderEventObserver,iEventObserver);
	static iRenderEventObserver* New(iViewModule *vm = 0);

	inline bool IsCancelled() const { return mCancelled; }
	inline float GetRenderTime() const { return mRenderTime; }

	void SetInteractive(bool s){ mInteractive = s; }
	bool IsInteractive() const { return mInteractive; }

	static void BlockRenderEventObservers(bool s) { mTheseAreBlocked = s; }
	//static bool RenderEventObserversBlocked(){ return mTheseAreBlocked; }

	virtual void PostFinished() = 0; // can be manually driven

protected:

	iRenderEventObserver(iViewModule *vm);
	~iRenderEventObserver();
	
	vtkTimerLog* Timer() const { return mRenderTimer; }

	virtual void ExecuteBody(vtkObject *caller, unsigned long event, void *data);

	virtual void OnStart() = 0;
	virtual void OnFinish() = 0;
	virtual bool CheckAbort() = 0;

private:

	vtkTimerLog *mRenderTimer;
	float mRenderTime;
	static bool mTheseAreBlocked;
	bool mCancelled, mInProgress, mInteractive;
	long mMasterThreadId;
};


class iPickEventObserver : public iEventObserver, public iViewModuleComponent
{
	
public:
	
	vtkTypeMacro(iPickEventObserver,iEventObserver);
	static iPickEventObserver* New(iViewModule *vm = 0);

protected:

	iPickEventObserver(iViewModule *vm);

	virtual void ExecuteBody(vtkObject *caller, unsigned long event, void *data);
	virtual void OnStart() = 0;
	virtual void OnFinish() = 0;

};

#endif // IVIEWEVENTOBSERVERS_H

