//LICENSE A


//
//  A container class that collects all ViewSubjects of the same type but accepting different DataType(s).
//
#ifndef IVIEWOBJECT_H
#define IVIEWOBJECT_H


#include "iobject.h"
#include "iviewmodulecomponent.h"


#include "iarray.h"

class iDataInfo;
class iSyncRequest;
class iViewInstance;
class iViewSubject;


//
//  Main class
//
class iViewObject : public iObject, public iViewModuleComponent
{
	
public:
	
	vtkTypeMacro(iViewObject,iObject);
	static iViewObject* New(iViewModule *parent, const iString &fname, const iString &sname);

	int GetNumberOfSubjects() const;
	iViewSubject* GetSubject(int id) const;
	iViewSubject* GetSubject(const iDataType &type) const;

	void Reset();
	float GetMemorySize() const;

	virtual void BecomeClone(iViewObject *v);

	virtual void WriteState(iString &s) const;
	virtual void RegisterWithHelpFactory() const;
	virtual const iObject* FindByName(const iString &name) const;

	virtual int DepthPeelingRequest() const;

	//
	//  Constructor helpers used by extensions
	//
	void AddSubject(iViewSubject *subject);

	virtual bool IsUsingData(const iDataType &type, bool onlyprimary) const;
	void SyncWithData(const iSyncRequest &request);

protected:
	
	virtual ~iViewObject();

	virtual void WriteHead(iString &s) const;

private:

	iViewObject(iViewModule *parent, const iString &fname, const iString &sname);

	//
	//  Members
	//
	iArray<iViewSubject*> mSubjects;

	static iViewObject* New(){ return 0; }
};

#endif // IVIEWOBJECT_H
