//LICENSE A


//
//  This is base class for all parallellizable pipelines
//

#ifndef IVIEWSUBJECTPARALLELPIPELINE_H
#define IVIEWSUBJECTPARALLELPIPELINE_H


#include "iviewsubjectpipeline.h"
#include "iparallelworker.h"


#include "iarray.h"

class iViewSubjectPipelineDataManager;


class iViewSubjectParallelPipeline : public iViewSubjectPipeline, protected iParallelWorker
{

public:

	vtkTypeMacro(iViewSubjectParallelPipeline,iViewSubjectPipeline);
	static iViewSubjectParallelPipeline* New(iViewInstance *owner = 0, int numInputs = 0, int id = 0);

	virtual void UpdateContents();
	virtual void UpdateContents(const iViewSubjectPipeline::Key &key);

	void Modified();
	void Reconfigure();

	inline iViewSubjectPipelineDataManager* GetDataManager() const { return mDataManager; }

	virtual float GetMemorySize();
	virtual void RemoveInternalData();

protected:

	iViewSubjectParallelPipeline(iViewInstance *owner, int numInputs, int id);
	virtual ~iViewSubjectParallelPipeline();

	virtual int FillInputPortInformation(int port, vtkInformation *info);
	virtual int FillOutputPortInformation(int port, vtkInformation *info);

	virtual void ProvideInfo();
	virtual void ProvideOutput(); 
	virtual void UpdateReplicas();

	virtual int ExecuteStep(int step, iParallel::ProcessorInfo &p);

private:

	void ConfigurePipelines(vtkDataSet *input);

	const int mId;
	unsigned long mConfigureTime;
	iViewInstance *mOwner;
	iViewSubjectPipelineDataManager *mDataManager;
	iArray<iViewSubjectPipeline*> mWorkPipelines;
};

#endif  // IVIEWSUBJECTPARALLELPIPELINE_H

