//LICENSE A


#include "iviewsubjectpipelinedatamanager.h"


#include "ierror.h"
#include "iviewsubject.h"

#include <vtkDataSet.h>

//
//  Templates
//
#include "iarray.tlh"


//
//  iViewSubjectPipelineDataDistributor class
//
iViewSubjectPipelineDataDistributor::iViewSubjectPipelineDataDistributor(iViewSubjectPipelineDataManager *manager, const iString &type)
{
	IASSERT(manager);
	
	mManager = manager;
	mType = type;
}


iViewSubjectPipelineDataDistributor::~iViewSubjectPipelineDataDistributor()
{
}


bool iViewSubjectPipelineDataDistributor::DistributeData(vtkDataSet *globalInput, iArray<vtkDataSet*> &localInputs)
{
	if(globalInput==0 || !globalInput->IsA(mType.ToCharPointer())) return false;

	return this->DistributeDataBody(globalInput,localInputs);
}


//
//  iViewSubjectPipelineDataCollector class
//
iViewSubjectPipelineDataCollector::iViewSubjectPipelineDataCollector(iViewSubjectPipelineDataManager *manager, const iString &type)
{
	IASSERT(manager);
	
	mManager = manager;
	mType = type;
}


iViewSubjectPipelineDataCollector::~iViewSubjectPipelineDataCollector()
{
}


bool iViewSubjectPipelineDataCollector::CollectData(iArray<vtkDataSet*> &localOutputs, vtkDataSet *globalOutput)
{
	if(globalOutput==0 || !globalOutput->IsA(mType.ToCharPointer())) return false;

	int i;
	for(i=0; i<localOutputs.Size(); i++)
	{
		if(localOutputs[i]==0 || !localOutputs[i]->IsA(mType.ToCharPointer())) return false;
	}

	return this->CollectDataBody(localOutputs,globalOutput);
}


//
//  iViewSubjectPipelineDataManager class
//
iViewSubjectPipelineDataManager* iViewSubjectPipelineDataManager::New(iViewInstance *owner)
{
	return new iViewSubjectPipelineDataManager(owner);
}


iViewSubjectPipelineDataManager::iViewSubjectPipelineDataManager(iViewInstance *owner) : iDataHandler(owner)
{
	mOwner = owner;
}


iViewSubjectPipelineDataManager::~iViewSubjectPipelineDataManager()
{
	while(mDistributors.Size() > 0) mDistributors.RemoveLast()->Delete();
	while(mCollectors.Size() > 0) mCollectors.RemoveLast()->Delete();
}


void iViewSubjectPipelineDataManager::DistributeData(vtkDataSet *globalInput, iArray<vtkDataSet*> &localInputs)
{
	int i;
	for(i=0; i<mDistributors.Size(); i++)
	{
		if(mDistributors[i]->DistributeData(globalInput,localInputs)) return;
	}

	IBUG_WARN("Unable to find a helper to distribute data in parallel. IFrIT will fall back to serial execution.");

	//
	//  Channel all data into the master pipeline
	//
	for(i=0; i<localInputs.Size(); i++)
	{
		localInputs[i] = globalInput->NewInstance();
	}
	localInputs[0]->ShallowCopy(globalInput);
}


void iViewSubjectPipelineDataManager::CollectData(iArray<vtkDataSet*> &localOutputs, vtkDataSet *globalOutput)
{
	int i;
	for(i=0; i<mCollectors.Size(); i++)
	{
		if(mCollectors[i]->CollectData(localOutputs,globalOutput)) return;
	}

	IBUG_WARN("Misconfiguration in iViewSubjectPipelineDataManager: unable to find a helper to collect the data.");

	//
	//  Use the master pipeline output
	//
	globalOutput->ShallowCopy(localOutputs[0]);
}


const iViewSubjectPipelineDataDistributor* iViewSubjectPipelineDataManager::GetDistributor(const iString &type) const
{
	int i;
	for(i=0; i<mDistributors.Size(); i++)
	{
		if(mDistributors[i]->GetType() == type) return mDistributors[i];
	}
	return 0;
}


const iViewSubjectPipelineDataCollector* iViewSubjectPipelineDataManager::GetCollector(const iString &type) const
{
	int i;
	for(i=0; i<mCollectors.Size(); i++)
	{
		if(mCollectors[i]->GetType() == type) return mCollectors[i];
	}
	return 0;
}


void iViewSubjectPipelineDataManager::AddDistributor(iViewSubjectPipelineDataDistributor *distributor)
{
	if(distributor != 0) mDistributors.Add(distributor);
}


void iViewSubjectPipelineDataManager::AddCollector(iViewSubjectPipelineDataCollector *collector)
{
	if(collector != 0) mCollectors.Add(collector);
}

