//LICENSE A

//
//  This class provides all functionality for writing images and animations
//
#ifndef IWRITER_H
#define IWRITER_H


#include "iobject.h"
#include "iviewmodulecomponent.h"


#include "istring.h"

#include <vtkSetGet.h>

class iGenericMultiImageMovieWriter;
class iStereoImageArray;

class vtkImageWriter;

namespace iParameter
{
	namespace ImageFormat
	{
		const int PNG =		0;
		const int JPG =		1;
		const int PNM =		2;
		const int BMP =		3;
		const int TIF =		4;
		const int EPS =		5;

		inline bool IsValid(int m){ return m>=0 && m<=5; }
	};

	namespace MovieOutput
	{
		const int Image =	0;
		const int MPEG2 =	1;
		const int AVI   =	2;

		inline bool IsValid(int m){ return m>=0 && m<=2; }
	};
};


class iWriter : public iObject, public iViewModuleComponent
{
	
public:

	vtkTypeMacro(iWriter,iObject);
	static iWriter* New(iViewModule *vm = 0);

	iObjectPropertyMacro1S(ImageFormat,Int);
	iObjectPropertyMacro1S(MovieOutput,Int);
	iObjectPropertyMacro1S(PostScriptPaperFormat,Int);
	iObjectPropertyMacro1S(PostScriptOrientation,Int);

	iObjectPropertyMacro2S(ImageRootName,String);
	iObjectPropertyMacro2S(MovieRootName,String);
	iString GetImageFileName();
	iString GetMovieFileName();

	iObjectPropertyMacroA(Write);

	//
	//  Writing functionality
	//
	void WriteImageFrame(const iStereoImageArray& images);
	void StartMovie();
	void WriteMovieFrame(const iStereoImageArray& images);
	void FinishMovie();

protected:
	
	virtual ~iWriter();

private:

	iWriter(iViewModule *vm, const iString &fname, const iString &sname);

	//
	//  VTK classes
	//
	iGenericMultiImageMovieWriter *mMovieWriter;  // for stereo movies
	vtkImageWriter *mImageWriter;

	static int mImageIndex, mMovieIndex;
	static iString mImageRootName, mMovieRootName;
};

#endif // IWRITER_H

