//LICENSE A
  

#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "ibgrenderwindowsubject.h"


#include "ierror.h"
#include "iviewmodule.h"

#include "iggmainwindow.h"
#include "iggrenderwindow.h"
#include "iggrenderwindowinteractor.h"
#include "iggshell.h"


bool ibgRenderWindowSubject::RenderOnFocus = false;


ibgRenderWindowSubject::ibgRenderWindowSubject(iggRenderWindow *owner) : ibgWindowSubject(owner?iRequiredCast<iggShell>(INFO,owner->GetViewModule()->GetShell()):0)
{
	IASSERT(owner);

	mOwner = owner;
	mInteractor = 0;

	mInRender = false;
}


ibgRenderWindowSubject::~ibgRenderWindowSubject()
{
}


void ibgRenderWindowSubject::Render()
{
	if(mInRender || RenderOnFocus || !mOwner->IsInitialized()) return;
	mInRender = true;
	mOwner->Render();
	mInRender = false;
}


void ibgRenderWindowSubject::OnMouseMoveBody(int x, int y, int b) 
{
	if(!this->IsBlocked() && this->IsInteractorPresent()) mInteractor->OnMouseMove(x,y,b);
}


void ibgRenderWindowSubject::OnMousePressBody(int x, int y, int b) 
{
	if(!this->IsBlocked() && this->IsInteractorPresent()) mInteractor->OnMousePress(x,y,b);
}


void ibgRenderWindowSubject::OnMouseReleaseBody(int x, int y, int b) 
{
	if(!this->IsBlocked() && this->IsInteractorPresent()) mInteractor->OnMouseRelease(x,y,b);
}


void ibgRenderWindowSubject::OnKeyPressBody(char key, int b) 
{
	if(!this->IsBlocked() && this->IsInteractorPresent()) mInteractor->OnKeyPress(key,b);
}


void ibgRenderWindowSubject::OnTimerBody() 
{
	if(!this->IsBlocked() && this->IsInteractorPresent()) mInteractor->OnTimer();
}


void ibgRenderWindowSubject::OnWindowFocusInBody()
{
	if(!this->IsBlocked())
	{
		if(RenderOnFocus)
		{
			mOwner->Render();
		}

		if(mOwner->GetMainWindow()!=0)
		{
			mOwner->GetMainWindow()->OnRenderWindowFocusIn(mOwner->GetViewModule()->GetWindowNumber());
			if(this->IsInteractorPresent()) mInteractor->OnExpose();
		}
	}
}


void ibgRenderWindowSubject::OnWindowFocusOutBody()
{
	if(!this->IsBlocked() && mOwner->GetMainWindow()!=0)
	{
		mOwner->GetMainWindow()->OnRenderWindowFocusOut(mOwner->GetViewModule()->GetWindowNumber());
		if(this->IsInteractorPresent()) mInteractor->OnExpose();
	}
}


void ibgRenderWindowSubject::OnWindowEnterBody()
{
	if(!this->IsBlocked() && mOwner->GetMainWindow()!=0)
	{
		mOwner->GetMainWindow()->OnRenderWindowEnter(mOwner->GetViewModule()->GetWindowNumber());
		if(this->IsInteractorPresent()) mInteractor->OnEnter();
	}
}


void ibgRenderWindowSubject::OnWindowLeaveBody()
{
	if(!this->IsBlocked() && mOwner->GetMainWindow()!=0)
	{
		mOwner->GetMainWindow()->OnRenderWindowLeave(mOwner->GetViewModule()->GetWindowNumber());
		if(this->IsInteractorPresent()) mInteractor->OnLeave();
	}
}


void ibgRenderWindowSubject::OnWindowResizeBody()
{
	if(!this->IsBlocked() && mOwner->GetMainWindow()!=0)
	{
		mOwner->GetMainWindow()->OnRenderWindowResize(mOwner->GetViewModule()->GetWindowNumber());
		mOwner->Modified();
	}
}


void ibgRenderWindowSubject::OnWindowMoveBody()
{
	if(!this->IsBlocked() && mOwner->GetMainWindow()!=0)
	{
		mOwner->GetMainWindow()->OnRenderWindowMove(mOwner->GetViewModule()->GetWindowNumber());
		mOwner->Modified();
	}
}


bool ibgRenderWindowSubject::IsInteractorPresent()
{
	if(mInteractor == 0)
	{
		mInteractor = iRequiredCast<iggRenderWindowInteractor>(INFO,mOwner->GetInteractor());
	}
	return mInteractor != 0;
}

#endif
