//LICENSE A
  

#ifndef IBGRENDERWINDOWSUBJECT_H
#define IBGRENDERWINDOWSUBJECT_H


#include "ibgwindowsubject.h"


class iggMainWindow;
class iggRenderWindow;
class iggRenderWindowInteractor;


class ibgRenderWindowSubject : public ibgWindowSubject
{

	friend class iggMainWindow;
	friend class iggRenderWindow;
	friend class iggRenderWindowInteractor;

public:

	static bool RenderOnFocus;

protected:

	ibgRenderWindowSubject(iggRenderWindow *owner);
	virtual ~ibgRenderWindowSubject();

	//
	//  Shell-specific
	//
	virtual void UpdateActiveStatus(bool last) = 0;

	//
	//  RenderWindow methods
	//
	virtual void Initialize(bool show) = 0;
	virtual void Finalize() = 0;
	virtual void SwapBuffers() = 0;

	virtual void SetBorder(bool) = 0;
	virtual void SetFullScreen(bool) = 0;
	virtual void MakeCurrent() = 0;

	virtual void* GetDisplay() const = 0;
	virtual void* GetContext() const = 0;
	virtual void* GetWindowId() const = 0;
	virtual void SetWindowName(const iString &s) = 0;

	virtual void RenderIntoMemory(iImage &) = 0;
	virtual bool IsReadyForDumpingImage() = 0;

	virtual bool IsDirect() const = 0;
	virtual bool IsCurrent() const = 0;

	virtual void RequestStereo(bool s) = 0;

	//
	//  Render the scene
	//
	void Render();

	//
	//  RenderWindowInteractor methods
	//
	virtual int InternalCreateTimer(int timerId, int timerType, unsigned long duration) = 0;
	virtual int InternalDestroyTimer(int platformTimerId) = 0;

	void OnMouseMoveBody(int x, int y, int b);
	void OnMousePressBody(int x, int y, int b);
	void OnMouseReleaseBody(int x, int y, int b);
	void OnKeyPressBody(char k, int b);
	void OnTimerBody();

 	void OnWindowFocusInBody();
	void OnWindowFocusOutBody();
	void OnWindowEnterBody();
	void OnWindowLeaveBody();
	void OnWindowResizeBody();
    void OnWindowMoveBody();

	bool IsInteractorPresent();

	iggRenderWindow *mOwner;
	iggRenderWindowInteractor *mInteractor;

private:

	bool mInRender;
};

#endif //IBGRENDERWINDOWSUBJECT_H
