//LICENSE A

//
//  Generic selection box subject
//
#ifndef IBGWIDGETSELECTIONBOXSUBJECT_H
#define IBGWIDGETSELECTIONBOXSUBJECT_H


#include "ibgwidgetsubject.h"


class iImage;
class iString;


//
//  Abstract owner class
//
class ibgWidgetSelectionBoxSubject : public ibgWidgetSubject
{

public:

	virtual ~ibgWidgetSelectionBoxSubject();

	//
	//  Decorator functions
	//
	virtual int GetValue() const = 0;
	virtual void SetValue(int v) = 0;

protected:

	ibgWidgetSelectionBoxSubject(iggWidget *owner);
};


//
//  Spin box
//
class ibgWidgetSpinBoxSubject : public ibgWidgetSelectionBoxSubject
{

public:

	virtual ~ibgWidgetSpinBoxSubject();

	virtual void SetStretch(int title, int box) = 0;

	//
	//  Decorator functions
	//
	virtual void SetFirstEntryText(const iString &text) = 0;
	virtual void SetRange(int min, int max) = 0;
	virtual void SetStep(int step) = 0;
	virtual int Count() = 0;

protected:

	ibgWidgetSpinBoxSubject(iggWidget *owner, int min, int max, const iString &title, int step);
};


//
//  Combo box
//
class ibgWidgetComboBoxSubject : public ibgWidgetSelectionBoxSubject
{

public:

	virtual ~ibgWidgetComboBoxSubject();

	virtual void SetStretch(int title, int box) = 0;

	//
	//  Decorator functions
	//
	virtual const iString GetText(int index = -1) const = 0;
	virtual void SetText(const iString &text) = 0;
	virtual void InsertItem(const iString &text, int index = -1) = 0;
	virtual void InsertItem(const iImage &icon, const iString &text, int index = -1) = 0;
	virtual void SetItem(const iString &text, int index, bool vis) = 0;
	virtual void RemoveItem(int index) = 0;
	virtual int Count() = 0;
	virtual void Clear() = 0;
	virtual bool DoesContentFit() const = 0;

protected:

	ibgWidgetComboBoxSubject(iggWidget *owner, const iString &title, bool bold);
};


//
//  Radio button box
//
class ibgWidgetRadioBoxSubject : public ibgWidgetSelectionBoxSubject
{

public:

	virtual ~ibgWidgetRadioBoxSubject();

	//
	//  Decorator functions
	//
	virtual void InsertItem(const iString &text, int index = -1) = 0;
	virtual void InsertItem(const iImage &icon, const iString &text, int index = -1) = 0;
	virtual void SetItem(const iString &text, int index, bool vis) = 0;
	virtual void RemoveItem(int index) = 0;
	virtual int Count() = 0;

protected:

	ibgWidgetRadioBoxSubject(iggWidget *owner, int cols, const iString &title);
};

#endif  // IBGWIDGETSELECTIONBOXSUBJECT_H

