//LICENSE A

//
//  Toolkit-independent base for window subject - classes that implement basic
//  window functions like resizing and showing/hiding
//

#ifndef IBGWINDOWHELPER_H
#define IBGWINDOWHELPER_H


#include "ibgelement.h"


class iImage;
class iString;


class ibgWindowHelper : public ibgElement
{

	friend class iggShell;

public:

	//
	//  Save and restore geometries for docking
	//
	void SaveWindowGeometry();
	void RestoreWindowGeometry();

	//
	//  Inheritable interface
	//
	virtual void SetWindowGeometry(int wg[4], bool force) = 0;
	virtual void GetWindowGeometry(int wg[4], bool force) const = 0;

	virtual void GetFrameSize(int &w, int &h) const = 0;

	virtual void Show(bool s) = 0;
	virtual bool IsVisible() const = 0;

	virtual void ShowAsWindow() = 0;
	virtual void ShowAsIcon() = 0;

	virtual void RestoreDecoration() = 0;

	inline static int GetFontOffset(){ return mFontOffset; }

protected:

	ibgWindowHelper(const iImage *icon, const iString &title, bool disableClose);

	static void ChangeFontSize(int s);

	const iImage *mIcon;
	const iString &mTitle;

	bool mDisableClose;
	int mSavedGeometry[4];

	static int mFontOffset;
};

#endif  // IBGWINDOWHELPER_H

