//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_CL)


#include "iclviewmoduleeventobservers.h"


#include "idataformatter.h"
#include "ierror.h"
#include "ipicker.h"
#include "iviewmodule.h"

#include "iclshell.h"

#include <vtkTimerLog.h>


//
//  iclRenderEventObserver class
//
iclRenderEventObserver::iclRenderEventObserver(iViewModule *vm) : iRenderEventObserver(vm)
{
	mStarted = false;
}


iclRenderEventObserver::~iclRenderEventObserver()
{
}


bool iclRenderEventObserver::CheckAbort()
{
	if(this->GetViewModule()->GetShell()->IsRunning())
	{
		this->Timer()->StopTimer();
		if(this->Timer()->GetElapsedTime() > 2.0) mStarted = true;
	}

	if(mStarted)
	{
		cout << "\rRendering...  ";
		switch(wSymbol)
		{
		case 0:
			{
				cout << "|";
				break;
			}
		case 1:
			{
				cout << "/";
				break;
			}
		case 2:
			{
				cout << "-";
				break;
			}
		case 3:
			{
				cout << "\\";
				break;
			}
		}
		wSymbol++; if(wSymbol > 3) wSymbol = 0;
		return false;
	}
	else return false;
}


void iclRenderEventObserver::OnStart()
{
	wSymbol = 0;
	mStarted = false;
}


void iclRenderEventObserver::OnFinish()
{
	if(mStarted)
	{
		mStarted = false;
		cout << "\bdone." << endl;
	}

	iclShell *shell = iRequiredCast<iclShell>(INFO,this->GetViewModule()->GetShell());
	if(shell->GetTrace())
	{
		iString str;

		if(shell->GetNumberOfViewModules() > 1)
		{
			str += "window #" + iString::FromNumber(this->GetViewModule()->GetWindowNumber()) + ": ";
		}
		str += "render time: " + iString::FromNumber(this->GetRenderTime()) + " secs.";

		cout << str.ToCharPointer() << endl;
	}
}


void iclRenderEventObserver::PostFinished()
{
	this->OnFinish();
}

//
//  iclPickEventObserver class
//
void iclPickEventObserver::OnStart()
{
}


void iclPickEventObserver::OnFinish()
{
	iPicker *picker = this->GetViewModule()->GetPicker();
	if(picker->GetObjectName().IsEmpty())
	{
		//
		//  Nothing is picked.
		//
		return;
	}

	cout << "Picker info:" << endl;

	cout << "Object: " << picker->GetObjectName().ToCharPointer() << endl;

	cout << "Location: " << 
		picker->GetPosition().BoxValue(0) << "," <<
		picker->GetPosition().BoxValue(1) << "," <<
		picker->GetPosition().BoxValue(2) << endl;

	int i, lnmax, lvmax, lumax, lmax;
	iString name, valu, unit, s;

	lnmax = lvmax = lumax = 0;
	for(i=0; i<picker->GetDataFormatter()->GetNumReportLines(); i++)
	{
		picker->GetDataFormatter()->GetReport(i,name,valu,unit);
		if(lnmax < name.Length()) lnmax = name.Length();
		if(lvmax < valu.Length()) lvmax = valu.Length();
		if(lumax < unit.Length()) lumax = unit.Length();
	}
	
	lmax = lnmax;
	if(lmax < lvmax) lmax = lvmax;
	if(lmax < lumax) lmax = lumax;
	for(i=0; i<lmax; i++) s += " ";
	
	for(i=0; i<picker->GetDataFormatter()->GetNumReportLines(); i++)
	{
		picker->GetDataFormatter()->GetReport(i,name,valu,unit);
		cout << 
			name.ToCharPointer() << s.Part(0,lnmax-name.Length()).ToCharPointer() << "  " <<
			valu.ToCharPointer() << s.Part(0,lvmax-valu.Length()).ToCharPointer() << "  " <<
			unit.ToCharPointer() << s.Part(0,lnmax-unit.Length()).ToCharPointer() << endl;
	}
	cout << "(Pick nothing to hide the picked point marker.)" << endl;
}

#endif
