//LICENSE A

//
//  A factory that creates various toolkit-specific subjects
//

#ifndef IGGSUBJECTFACTORY_H
#define IGGSUBJECTFACTORY_H


class iImage;
class iShell;
class iString;

class iggDialog;
class iggExtensionWindow;
class iggFrame;
class iggFrameBook;
class iggFrameFlip;
class iggFrameScroll;
class iggFrameSplit;
class iggMainWindow;
class iggMenuWindow;
class iggRenderWindow;
class iggShell;
class iggWidget;
class iggWidgetDrawArea;
class iggWidgetHelpBrowser;
class iggWidgetTextEditor;

class ibgDialogSubject;
class ibgExtensionWindowSubject;
class ibgFrameSubject;
class ibgFrameBookSubject;
class ibgFrameFlipSubject;
class ibgFrameScrollSubject;
class ibgFrameSplitSubject;
class ibgMainWindowSubject;
class ibgMenuWindowSubject;
class ibgRenderWindowSubject;
class ibgShellSubject;
class ibgWidgetButtonSubject;
class ibgWidgetColorSelectionSubject;
class ibgWidgetComboBoxSubject;
class ibgWidgetDisplayAreaSubject;
class ibgWidgetDrawAreaSubject;
class ibgWidgetEntrySubject;
class ibgWidgetHelpBrowserSubject;
class ibgWidgetMultiImageDisplayAreaSubject;
class ibgWidgetProgressBarSubject;
class ibgWidgetRadioBoxSubject;
class ibgWidgetSpinBoxSubject;
class ibgWidgetTrackBallSubject;
class ibgWidgetTextEditorSubject;
class ibgWindowSubject;


class iggSubjectFactory
{
	
public:

	static ibgDialogSubject*						CreateDialogSubject(iggDialog *owner, const ibgWindowSubject *base, unsigned int mode, const iImage *icon, const iString &title);

	static ibgExtensionWindowSubject*				CreateExtensionWindowSubject(iggExtensionWindow *owner);

	static ibgFrameSubject*							CreateFrameSubject(iggFrame *owner, int cols);
	static ibgFrameBookSubject*						CreateFrameBookSubject(iggFrameBook *owner, bool withFeedback);
	static ibgFrameFlipSubject*						CreateFrameFlipSubject(iggFrameFlip *owner, bool expanding);
	static ibgFrameScrollSubject*					CreateFrameScrollSubject(iggFrameScroll *owner, bool withHor, bool withVer);
	static ibgFrameSplitSubject*					CreateFrameSplitSubject(iggFrameSplit *owner, bool hor);

	static ibgMainWindowSubject*					CreateMainWindowSubject(iggMainWindow *owner, int cols);
	static ibgMenuWindowSubject*					CreateMenuWindowSubject(iggMenuWindow *owner, const iImage *icon, const iString &title);

	static ibgRenderWindowSubject*					CreateRenderWindowSubject(iggRenderWindow *owner);

	static ibgShellSubject*							CreateShellSubject(iggShell *owner);

	static ibgWidgetButtonSubject*					CreateWidgetButtonSubject(iggWidget *owner, int type, const iString &text, int slot, int minsize = 0);
	static ibgWidgetColorSelectionSubject*			CreateWidgetColorSelectionSubject(iggWidget *owner);
	static ibgWidgetComboBoxSubject*				CreateWidgetComboBoxSubject(iggWidget *owner, const iString &title, bool bold = false);
	static ibgWidgetDisplayAreaSubject*				CreateWidgetDisplayAreaSubject(iggWidget *owner, const iString &text);
	static ibgWidgetDrawAreaSubject*				CreateWidgetDrawAreaSubject(iggWidgetDrawArea *owner, bool interactive);
	static ibgWidgetEntrySubject*					CreateWidgetEntrySubject(iggWidget *owner, bool slider, int numdig, const iString &label);
	static ibgWidgetHelpBrowserSubject*				CreateWidgetHelpBrowserSubject(iggWidgetHelpBrowser *owner);
	static ibgWidgetMultiImageDisplayAreaSubject*	CreateWidgetMultiImageDisplayAreaSubject(iggWidget *owner);
	static ibgWidgetProgressBarSubject*				CreateWidgetProgressBarSubject(iggWidget *owner);
	static ibgWidgetRadioBoxSubject*				CreateWidgetRadioBoxSubject(iggWidget *owner, int cols, const iString &title);
	static ibgWidgetSpinBoxSubject*					CreateWidgetSpinBoxSubject(iggWidget *owner, int min, int max, const iString &title, int step);
	static ibgWidgetTextEditorSubject*				CreateWidgetTextEditorSubject(iggWidgetTextEditor *owner, unsigned int mode);
	static ibgWidgetTrackBallSubject*				CreateWidgetTrackBallSubject(iggWidget *owner, bool followCamera);

	static const iString GetIncludedShells();

private:

	static const iString qt, fx;
};

#endif // IGGSUBJECTFACTORY_H
