//LICENSE A

//
//  A base class for generic dialogs (toolkit-independent)
//

#ifndef IGGDIALOG_H
#define IGGDIALOG_H


#include "iggrenderingelement.h"


#include "iarray.h"


class iImage;
class iString;

struct iHelpData;
class iHelpDataBuffer;

class iggFrame;
class iggMainWindow;
class iggMenuWindow;

class ibgDialogSubject;
class ibgWindowSubject;


namespace iParameter
{
	//
	//  Dialog modes masks
	//
	namespace DialogFlag
	{
		const unsigned int Blocking = 1U;
		const unsigned int WithEventLoop = 2U;
		const unsigned int Modal = 3U;
		const unsigned int NoTitleBar = 4U;
		const unsigned int Unattached = 8U;
	};
};


class iggDialog : public iggRenderingElement
{

	friend class iggSubjectFactory;

public:

	virtual ~iggDialog();

	inline ibgDialogSubject* GetSubject() const { return mSubject; }
	inline iggFrame* GetFrame() const { return mFrame; }

	void Show(bool s);
	bool IsVisible() const;
	void ResizeContents(int w, int h);

	void Close();
	virtual void UpdateDialog();

	iggMainWindow* GetMainWindow() const { return mMainWindow; }

	//
	//  Provides help for this dialog
	//
	virtual const iString& GetToolTip() const;
	iString GetBaloonHelp() const;

#ifdef I_DEBUG
	static void FlashAllDialogs();
#endif

protected:

	iggDialog(iggShell *shell, unsigned int mode, const iImage *icon, const iString &title, const char *helpkey, int cols, const char *closetext = "");
	iggDialog(const iggMenuWindow *mw, unsigned int mode, const iImage *icon, const iString &title, const char *helpkey, int cols, const char *closetext = "");
	iggDialog(const iggDialog *pd, unsigned int mode, const iImage *icon, const iString &title, const char *helpkey, int cols, const char *closetext = "");

	virtual void ShowBody(bool s);
	virtual bool CanBeClosed();
	void AttachSubject(ibgDialogSubject *subject); //  this is a dummy for factory use

	bool ImmediateConstruction() const; 
	void CompleteConstruction();
	virtual void CompleteConstructionBody() = 0;

	bool mCompleteConstructionCalled;
	ibgDialogSubject *mSubject;
	iggMainWindow *mMainWindow;
	iggFrame *mFrame;
	unsigned int mMode;
	iHelpDataBuffer *mHelp;

private:

	void Define(const ibgWindowSubject *base, unsigned int mode, const iImage *icon, const iString &title, const char *helpkey, int cols, const char *closetext);

	//
	//  Registry
	//
	static iLookupArray<iggDialog*>& List();

	iggDialog(const iggDialog&); // Not implemented.
	void operator=(const iggDialog&);  // Not implemented.
};

#endif  // IGGDIALOG_H

