//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "iggdialogloadfile.h"


#include "idata.h"
#include "idatareader.h"
#include "ierror.h"
#include "isystem.h"
#include "iviewmodule.h"

#include "iggframe.h"
#include "igghandle.h"
#include "iggmainwindow.h"
#include "iggshell.h"
#include "iggshelleventobservers.h"
#include "iggwidgetmisc.h"
#include "iggwidgetprogressbar.h"


using namespace iParameter;


const bool iggDialogLoadFile::mIsMultiThread = false;


namespace iggDialogLoadFile_Private
{
	class ProgressBar : public iggWidgetProgressBar
	{

	public:

		ProgressBar(iggFrame *parent) : iggWidgetProgressBar(parent)
		{
		}

	protected:

		virtual void OnSetProgress()
		{
			this->GetMainWindow()->ProcessEvents();
		}
	};
};


using namespace iggDialogLoadFile_Private;


//
//  Main class
//
iggDialogLoadFile::iggDialogLoadFile(iggMainWindow *parent) : iggDialog(parent,mIsMultiThread?0U:DialogFlag::Blocking,0,"Loading file...",0,2,"Abort")
{
	if(this->ImmediateConstruction()) this->CompleteConstruction();
}


void iggDialogLoadFile::CompleteConstructionBody()
{
	mProgressBar = new ProgressBar(mFrame);
	mLabel = new iggWidgetTextLabel("",mFrame);

	mFrame->AddLine(new iggWidgetTextLabel("%bLoading file:",mFrame));
	mFrame->AddLine(mLabel,2);
	mFrame->AddLine(mProgressBar,2);
	if(mIsMultiThread)
	{
		mFrame->AddLine(new iggWidgetTextLabel("You can work with other windows while file is being loaded.",mFrame),2);
	}
	else
	{
		mFrame->AddLine(new iggWidgetTextLabel("Please wait while file is being loaded.",mFrame),2);
	}

	mFrame->SetColStretch(1,10);

	this->ResizeContents(300,200);
}


iggDialogLoadFile::~iggDialogLoadFile()
{
}


void iggDialogLoadFile::Start()
{
	if(!mCompleteConstructionCalled) this->CompleteConstruction();

	iRequiredCast<iggExecutionEventObserver>(INFO,this->GetShell()->GetExecutionEventObserver())->SetProgressBar(mProgressBar);

	iggDialog::Show(true);
	this->GetMainWindow()->BLOCK(true);

	mLabel->SetText("...");
}


void iggDialogLoadFile::OnFileStart(const iString &fname, const iDataType &type)
{
	mLabel->SetText(fname);
}


void iggDialogLoadFile::OnFileFinish(const iString &fname, const iDataInfo &info)
{
}


void iggDialogLoadFile::Finish()
{
	this->GetMainWindow()->AfterDataChanged();

	this->GetMainWindow()->BLOCK(false);
	iggDialog::Show(false);
}


bool iggDialogLoadFile::CanBeClosed()
{
	mProgressBar->Abort();
	return true;
}

#endif
