//LICENSE A

//
//  Window with extension pages (extension manager)
//

#ifndef IGGEXTENSIONWINDOW_H
#define IGGEXTENSIONWINDOW_H


#include "iggelement.h"


#include "iarray.h"

class iDataType;
class iImage;
class iShell;
class iString;

class iggAbstractExtension;
class iggExtensionCreator;
class iggExtensionCreatorPointer;
class iggExtensionWindowHelper;
class iggFrame;
class iggFrameFlip;
class iggMainWindow;
class iggPageData;

class ibgExtensionWindowSubject;


class iggExtensionWindow : public iggElement
{

	friend class iggAbstractExtension;
	friend class iggExtensionCreator;
	friend class iggMainWindow;

public:

	virtual ~iggExtensionWindow();

	//
	//  Broadcast to all extensions
	//
	void OpenBookPageByIndex(int index);
	void SetTabMode(int m);

	void PopulateFileMenu();
	void PopulateLocalDialogMenu();
	void PopulateGlobalDialogMenu();
	void PopulateFileToolBar(bool postonly);
	void PrePopulateFileToolBar();
	void PopulateShowToolBar();

	bool OnMenuBody(int id, bool on);

	void UpdateOnPick();
	void UpdateParticleWidgets(const iImage *icon = 0);
	void AddReloadingDataTypes(iggPageData *page);
	const iImage* GetSpecialParticleIcon(const iDataType &type) const;

	void UpdateOnDataType(const iDataType &type);

	//
	//  Special functions
	//
	bool IsEmpty() const;
	void SetPadding(bool s);
	void OnCloseAttempt();

	//
	//  Access components
	//
	inline ibgExtensionWindowSubject* GetSubject() const { return mSubject; }
	inline iggMainWindow* GetMainWindow() const { return mMainWindow; }

private:

	iggExtensionWindow(iggMainWindow *mw);
	void CompleteInitialization();

	void UpdateAll();

	static void AttachCreator(iggExtensionCreator *c);
	static iArray<iggExtensionCreatorPointer>& Creators();

	iggFrame *mFrames[3];
	iggFrameFlip *mMainPanel;
	iggMainWindow *mMainWindow;
	ibgExtensionWindowSubject *mSubject;
	iggExtensionWindowHelper *mHelper;
	iArray<iggAbstractExtension*> mExt;
};

#endif  // IGGEXTENSIONWINDOW_H

