//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "iggframeboxsize.h"


#include "ierror.h"
#include "iviewmodule.h"

#include "igghandle.h"
#include "iggshell.h"
#include "iggwidgetmisc.h"
#include "iggwidgetpropertycontrolbutton.h"
#include "iggwidgetpropertycontrollineedit.h"

#include "ibgwidgetbuttonsubject.h"

//
//  Templates (needed for some compilers)
//
#include "iarray.tlh"


using namespace iParameter;


namespace iggFrameBoxSize_Private
{
	
	class LineEdit : public iggWidgetPropertyControlFloatLineEdit
	{
		
	public:
		
		LineEdit(iggWidgetSimpleCheckBox *cb, const iString &label, const iggPropertyHandle &ph, iggFrameBoxSize *parent) : iggWidgetPropertyControlFloatLineEdit(label,ph,parent)
		{
			mRealParent = parent;
			mCheckBox = cb;
		}

	protected:

		virtual void UpdateWidgetBody()
		{
			if(mCheckBox->IsChecked())
			{
				this->Enable(false);
			}
			else
			{
				this->Enable(true);
				this->iggWidgetPropertyControlFloatLineEdit::UpdateWidgetBody();
			}
		}

		virtual void OnVoid1Body()
		{
			iggWidgetPropertyControlFloatLineEdit::OnVoid1Body();
			if(mRealParent != 0)
			{
				mRealParent->UpdatePositions();
			}
		}

	private:

		iggFrameBoxSize *mRealParent;
		iggWidgetSimpleCheckBox *mCheckBox;
	};
	

	class CheckBox : public iggWidgetSimpleCheckBox
	{
		
	public:
		
		CheckBox(const char* text, iggFrameBoxSize *parent) : iggWidgetSimpleCheckBox(text,parent)
		{
			mRealParent = parent;

			this->SetBaloonHelp("Specify whether to use OpenGL coordinates (box axes go from -1 to 1) or to set the size of the box (axes go from 0 to box size)");
		}

	protected:

		virtual void UpdateWidgetBody()
		{
			mSubject->SetDown(this->GetShell()->GetViewModule()->InOpenGLCoordinates());
		}

		virtual void OnChecked(bool s)
		{
			this->GetShell()->GetViewModule()->SetBoxSize(s?0.0:1.0);
			if(mRealParent != 0)
			{
				mRealParent->UpdatePositions();
			}
			this->Render();
		}

	private:

		iggFrameBoxSize *mRealParent;
	};
};


using namespace iggFrameBoxSize_Private;


iggFrameBoxSize::iggFrameBoxSize(iggFrame *parent) : iggFrame("Coordinates",parent,1)
{
	CheckBox *cb = new CheckBox("Use OpenGL coordinates",this);
	LineEdit *le = new LineEdit(cb,"Box size",iggPropertyHandle("BoxSize",this->GetShell()->GetViewModuleHandle()),this);
	cb->AddDependent(le);

	this->AddLine(cb);
	this->AddLine(le);
}


void iggFrameBoxSize::UpdatePositions()
{
	int i;
	iLookupArray<iggWidgetPropertyControlBase*> &list(iggWidgetPropertyControlBase::PositionList());

	for(i=0; i<list.Size(); i++)
	{
		list[i]->UpdateWidget();
	}
}

#endif
