//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "iggframehistogramcontrols.h"


#include "ierror.h"
#include "ihistogram.h"
#include "istretch.h"

#include "iggwidgetarea.h"
#include "iggwidgetmisc.h"
#include "iggwidgetotherbutton.h"


using namespace iParameter;


namespace iggFrameHistogramControls_Private
{
	class ShowInLogCheckBox : public iggWidgetSimpleCheckBox
	{

	private:

		iggFrameHistogramControls *mOwner;

	public:

		ShowInLogCheckBox(iggFrameHistogramControls *owner, iggFrame *parent) : iggWidgetSimpleCheckBox("Show Y-axis in log",parent)
		{
			mOwner = owner;

			this->SetBaloonHelp("Show histogram in log scale");
		}

	protected:

		virtual void UpdateWidgetBody()
		{
			this->SetChecked(mOwner->GetYLogScale());
		}

		virtual void OnChecked(bool s)
		{
			mOwner->SetYLogScale(s);
			mOwner->UpdateArea();
		}
	};

	class ShowInFullResolutionCheckBox : public iggWidgetSimpleCheckBox
	{

	private:

		iggFrameHistogramControls *mOwner;

	public:

		ShowInFullResolutionCheckBox(iggFrameHistogramControls *owner, iggFrame *parent) : iggWidgetSimpleCheckBox("Full resoluton (may be slow)",parent)
		{
			mOwner = owner;

			this->SetBaloonHelp("Show histogram with full resolution (slow for large data sets)");
		}

	protected:

		virtual void UpdateWidgetBody()
		{
			this->SetChecked(mOwner->GetFullResolution());
		}

		virtual void OnChecked(bool s)
		{
			mOwner->SetFullResolution(s);
			mOwner->UpdateArea();
		}
	};

	class StretchComboBox : public iggWidgetSimpleComboBox
	{

	private:

		iggFrameHistogramControls *mOwner;

	public:

		StretchComboBox(iggFrameHistogramControls *owner, iggFrame *parent) : iggWidgetSimpleComboBox("Stretch",parent)
		{
			mOwner = owner;
			
			int i = 0;
			while(!iStretch::GetName(i).IsEmpty())
			{
				this->InsertItem(iStretch::GetName(i++));
			}

			this->OnInt1Body(mOwner->GetStretchId());

			this->SetBaloonHelp("Set the stretch for X-axis","This box sets the stretch (scaling) for the horizontal axis.");
		}

	protected:

		virtual void UpdateWidgetBody()
		{
			this->SetValue(mOwner->GetStretchId());
		}

		void OnInt1Body(int s)
		{
			mOwner->SetStretchId(s);
			mOwner->UpdateArea();
		}
	};
};


using namespace iggFrameHistogramControls_Private;


//
//  Main class
//
iggFrameHistogramControls::iggFrameHistogramControls(bool withfull, bool withstretch, iggFrame *parent) : iggFrame(parent)
{
	mFullResolution = !withfull;
	mYLogScale = false;
	mStretchId = iStretch::Lin;

	mHistogramArea = 0;

	iggFrame *tmp = new iggFrame(this,3);
	mDisplayMin = new iggWidgetNumberLabel(tmp);
	mDisplayMin->SetAlignment(-1);
	mDisplayMax = new iggWidgetNumberLabel(tmp);
	mDisplayMin->SetAlignment(1);
	tmp->AddLine(mDisplayMin,new iggWidgetTextLabel("<-- range -->",tmp),mDisplayMax);
	tmp->SetColStretch(1,10);
	this->AddLine(tmp);

	tmp = new iggFrame(this,3);
	if(withstretch)
	{
		tmp->AddLine(new ShowInLogCheckBox(this,tmp),static_cast<iggWidget*>(0),new StretchComboBox(this,tmp));
	}
	else
	{
		tmp->AddLine(new ShowInLogCheckBox(this,tmp));
	}
	if(withfull)
	{
		tmp->AddLine(new ShowInFullResolutionCheckBox(this,tmp));
	}
	tmp->SetColStretch(1,10);
	this->AddLine(tmp);
}


void iggFrameHistogramControls::SetYLogScale(bool s)
{
	mYLogScale = s;
	this->UpdateWidget();
}


void iggFrameHistogramControls::SetStretchId(int s)
{
	mStretchId = s;
	this->UpdateWidget();
}


void iggFrameHistogramControls::SetFullResolution(bool s)
{
	mFullResolution = s;
	this->UpdateWidget();
}


void iggFrameHistogramControls::DisplayRange(float min, float max)
{
	mDisplayMin->SetNumber(min);
	mDisplayMax->SetNumber(max);
}


void iggFrameHistogramControls::Attach(iggWidgetHistogramArea *area)
{
	mHistogramArea = area;
}

void iggFrameHistogramControls::UpdateArea()
{
	if(mHistogramArea != 0) mHistogramArea->UpdateWidget();
}

#endif
