//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "iggframematerialproperties.h"


#include "ierror.h"
#include "imaterial.h"
#include "imaterialobject.h"
#include "iviewsubject.h"

#include "igghandle.h"
#include "iggwidgetpropertycontrolselectionbox.h"
#include "iggwidgetpropertycontrolslider.h"
#include "iggwidgetotherbutton.h"

//
//  Templates (needed for some compilers)
//
#include "iarray.tlh"
#include "iggwidgetpropertycontrolslider.tlh"


using namespace iParameter;

namespace iggFrameMaterialProperties_Private
{
	class MaterialHandle : public iggObjectHandle
	{

	public:

		MaterialHandle(const iggObjectHandle *parent) : iggObjectHandle(parent->GetShell())
		{
			mParent = parent;
		}

		virtual const iObject* Object(int mod = -1) const
		{
			iMaterialObject *obj = iRequiredCast<iMaterialObject>(INFO,const_cast<iObject*>(mParent->Object(mod)));
			return obj->GetMaterial();
		}

	private:

		const iggObjectHandle *mParent;
	};
};


using namespace iggFrameMaterialProperties_Private;


iggFrameMaterialProperties::iggFrameMaterialProperties(const iggObjectHandle *handle, iggFrame *parent) : iggFrame("Material properties",parent,2)
{
	IASSERT(handle);
	const iMaterialObject *sub = dynamic_cast<const iMaterialObject*>(handle->Object());
	if(sub == 0)
	{
		IBUG_FATAL("Widget iggFrameMaterialProperties is congifured incorrectly.");
	}

	mMaterialHandle = new MaterialHandle(handle); IERROR_CHECK_MEMORY(mMaterialHandle);

	iggWidgetPropertyControlRadioBox *sm = new iggWidgetPropertyControlRadioBox(3,"Shading mode",1,iggPropertyHandle("ShadingMode",mMaterialHandle),this);
	sm->InsertItem("Off");
	sm->InsertItem("Auto");
	sm->InsertItem("On");
	this->AddLine(sm);
	
	iggWidgetPropertyControlFloatSlider *slider;
	slider = new iggWidgetPropertyControlFloatSlider(0.0,1.0,20,0,3,false,false,"Ambient",iggPropertyHandle("Ambient",mMaterialHandle),this);
	slider->SetStretch(10,15);
	this->AddLine(slider,2);

	slider = new iggWidgetPropertyControlFloatSlider(0.0,1.0,20,0,3,false,false,"Diffuse",iggPropertyHandle("Diffuse",mMaterialHandle),this);
	slider->SetStretch(10,15);
	this->AddLine(slider,2);

	slider = new iggWidgetPropertyControlFloatSlider(0.0,1.0,20,0,3,false,false,"Specular",iggPropertyHandle("Specular",mMaterialHandle),this);
	slider->SetStretch(10,15);
	this->AddLine(slider,2);

	slider = new iggWidgetPropertyControlFloatSlider(0.0,1.0,20,0,3,false,false,"Specular power",iggPropertyHandle("SpecularPower",mMaterialHandle),this);
	slider->SetStretch(10,15);
	this->AddLine(slider,2);
}


iggFrameMaterialProperties::~iggFrameMaterialProperties()
{
	delete mMaterialHandle;
}

#endif
