//LICENSE A


#include "iconfigure.h"
#if ISHELL_INCLUDED(ISHELL_GG)


#include "iggrenderwindowobserver.h"


#include "ierror.h"
#include "iviewmodule.h"

#include "iggframe.h"
#include "iggshell.h"

#include <vtkRenderWindow.h>


//
//  Main class
//
iggRenderWindowObserver::iggRenderWindowObserver(iggFrame *parent, bool disabled)
{
	IASSERT(parent);
	mParent = parent;

	mParent->RegisterRenderWindowObserver(this);

	if(disabled)
	{
		mHelper = 0;
	}
	else
	{
		mHelper = new iggRenderWindowObserverHelper(this,parent); IERROR_CHECK_MEMORY(mHelper);
	}
}


iggRenderWindowObserver::~iggRenderWindowObserver()
{
	if(mHelper != 0)
	{
		mHelper->Detach();
		mHelper->Delete();
	}

	mParent->UnRegisterRenderWindowObserver(this);
}

void iggRenderWindowObserver::UpdateConnection()
{
	if(mHelper != 0) mHelper->UpdateConnection();
}


//
//  Helper class
//
iggRenderWindowObserverHelper::iggRenderWindowObserverHelper(iggRenderWindowObserver *owner, iggFrame *frame) : iEventObserver()
{
	IASSERT(frame);
	IASSERT(owner);

	mFrame = frame;
	mOwner = owner;
	mWindow = 0;
}


iggRenderWindowObserverHelper::~iggRenderWindowObserverHelper()
{
}


void iggRenderWindowObserverHelper::Detach()
{
	if(mWindow != 0) mWindow->RemoveObserver(this);
	mWindow = 0;
}


void iggRenderWindowObserverHelper::UpdateConnection()
{
	vtkRenderWindow *win = mFrame->GetShell()->GetViewModule()->GetRenderWindow();
	if(win!=0 && win!=mWindow)
	{
		this->Detach();
		mWindow = win;
		mWindow->AddObserver(EndEvent,this);
		mWindow->AddObserver(DeleteEvent,this);
	}
}


void iggRenderWindowObserverHelper::ExecuteBody(vtkObject *caller, unsigned long event, void *)
{
#ifndef I_NO_CHECK
	if(caller != mWindow)
	{
		IBUG_WARN("Internal check failed.");
	}
#endif
	if(caller == mWindow)
	{
		switch(event)
		{
		case EndEvent:
			{
				if(mFrame->IsVisible())
				{
					mOwner->OnRenderWindowModified();
				}
				break;
			}
		case DeleteEvent:
			{
				this->Detach();
				break;
			}
		}
	}
}

#endif
