//LICENSE A


//
//  A generic GUI shell
//

#ifndef IGGSHELL_H
#define IGGSHELL_H


#include "ishell.h"


class iDataReader;
class iPicker;

class iggDialog;
class iggDialogFlashWindow;
class iggExtensionWindow;
class iggMainWindow;
class iggMenuWindow;
class iggObjectHandle;
class iggPropertyHandle;
class iggRenderWindow;

class ibgShellSubject;


namespace iParameter
{
	namespace ViewModuleSelection
	{
		const int One		= 1;
		const int All		= 2;
		const int Clones	= 3;
	}
};


class iggShell : public iShell
{

	friend class iShell;

public:

	vtkTypeMacro(iggShell,iShell);

	inline iggMainWindow* GetMainWindow() const { return mMainWindow; }

	iViewModule* GetViewModule(int mod = -1) const;
	bool SetActiveViewModuleIndex(int v);
	int GetActiveViewModuleIndex() const;
	void SelectViewModules(iArray<iViewModule*> &arr, int selection);

	void GetDesktopDimensions(int &w, int &h) const;
	void ProcessEvents(bool sync = false);

	bool IsDesktopSmall(bool vert) const;
	bool CheckCondition(int flag) const;
	void SetCondition(int flag);

	inline iggObjectHandle* GetShellHandle() const { return mShellHandle; }
	inline iggObjectHandle* GetViewModuleHandle() const { return mViewModuleHandle; }
	inline iggObjectHandle* GetDataReaderHandle() const { return mDataReaderHandle; }

	iggObjectHandle* GetObjectHandle(const iString &obj);

	//ibgShellSubject* GetSubject() const { return mSubject; }

	//
	//  User notification
	//
	int PopupWindow(const iString &text, int type, const char *button0 = "Ok", const char *button1 = 0, const char *button2 = 0) const;
	int PopupWindow(const iggDialog *parent, const iString &text, int type, const char *button0 = "Ok", const char *button1 = 0, const char *button2 = 0) const;
	int PopupWindow(const iggMenuWindow *parent, const iString &text, int type, const char *button0 = "Ok", const char *button1 = 0, const char *button2 = 0) const;
	int PopupWindow(const iggRenderWindow *parent, const iString &text, int type, const char *button0 = "Ok", const char *button1 = 0, const char *button2 = 0) const;
	int PopupWindow(const iggExtensionWindow *parent, const iString &text, int type, const char *button0 = "Ok", const char *button1 = 0, const char *button2 = 0) const;

	//
	//  Interface for multi-threader
	//
	virtual void OnTimerEvent();

	//
	//  Factory methods for ViewModule
	//
	virtual vtkRenderWindow* CreateRenderWindow(iViewModule *vm, bool stereo) const;
	virtual vtkRenderWindowInteractor* CreateRenderWindowInteractor(iViewModule *vm) const;
	virtual iEventObserver* CreateViewModuleEventObserver(iParameter::ViewModuleObserver::Type type, iViewModule *vm) const;

protected:

	iggShell(iRunner *runner, bool threaded, const iString &type, int argc, const char **argv);
	virtual ~iggShell();

	virtual bool StartConstructor();
	virtual bool ConstructorBody();  
	virtual void DestructorBody();  
	virtual void FinishDestructor();
	virtual void StartBody();
	virtual void StopBody();

	virtual void OnInitStart();
	virtual void OnInitState();
	virtual void OnInitFinish();

	virtual void OnInitAtomBody(bool timed);
	virtual void OnLoadStateAtomBody(int prog);

	virtual bool OnCommandLineOption(const iString &sname, const iString& value);

	//
	//  Factory methods for itself
	//
	virtual iDataReaderObserver* CreateDataReaderObserver();
	virtual iEventObserver* CreateShellEventObserver(iParameter::ShellObserver::Type);
	virtual iOutputChannel* CreateOutputChannel();

	//
	//  Use as a callback when the active window chages
	//
	virtual void OnActiveWindowChange() const;

private:

	iggObjectHandle *mShellHandle, *mViewModuleHandle, *mDataReaderHandle;
	iGenericOrderedArray<iggObjectHandle*,iString> mHandles;

	ibgShellSubject *mSubject;
	iggMainWindow *mMainWindow;
	bool mShowFlashWindow, mStartDocked;
	int mConditionFlags;
	iggDialogFlashWindow *mFlashWindow;
	mutable int mActiveViewModuleIndex;
};

#endif  // IGGSHELL_H

