//LICENSE A

//
//  Qt-based selection box
//
#ifndef IQTWIDGETSELECTIONBOXSUBJECT_H
#define IQTWIDGETSELECTIONBOXSUBJECT_H


#include "iqt.h"

#ifdef IQT_3
#include <qbuttongroup.h>
#include <qwidget.h>
#define QRadioBox QButtonGroup
#elif defined(IQT_4)
#include <QtGui/QGroupBox>
#include <QtGui/QWidget>
#define QRadioBox QGroupBox
#else
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QWidget>
#define QRadioBox QGroupBox
#endif
#include "ibgwidgetselectionboxsubject.h"

class QButtonGroup;
class QComboBox;
class QLabel;
class QSpinBox;


//
//  Spin box
//
class iqtWidgetSpinBoxSubject : public QWidget, public ibgWidgetSpinBoxSubject
{

	friend class iggSubjectFactory;

public:

	virtual ~iqtWidgetSpinBoxSubject();

	virtual void SetStretch(int title, int box);

	//
	//  Decorator functions
	//
	virtual int GetValue() const;
	virtual void SetValue(int v);
	virtual void SetFirstEntryText(const iString &text);
	virtual void SetRange(int min, int max);
	virtual void SetStep(int step);
	virtual int Count();

protected:

	iqtWidgetSpinBoxSubject(iggWidget *owner, int min, int max, const iString &title, int step);

	QSpinBox *mBox;
	QLabel *mTitle;
};


//
//  Combo box
//
class iqtWidgetComboBoxSubject : public QWidget, public ibgWidgetComboBoxSubject
{

	friend class iggSubjectFactory;

public:

	virtual ~iqtWidgetComboBoxSubject();

	virtual void SetStretch(int title, int box);

	//
	//  Decorator functions
	//
	virtual int GetValue() const;
	virtual void SetValue(int v);
	virtual const iString GetText(int index = -1) const;
	virtual void SetText(const iString &text);
	virtual void InsertItem(const iString &text, int index = -1);
	virtual void InsertItem(const iImage &icon, const iString &text, int index = -1);
	virtual void SetItem(const iString &text, int index, bool vis);
	virtual void RemoveItem(int index);
	virtual int Count();
	virtual void Clear();
	virtual bool DoesContentFit() const;

protected:

	iqtWidgetComboBoxSubject(iggWidget *owner, const iString &title, bool bold);

	QComboBox *mBox;
	QLabel *mTitle;
};


//
//  Radio button box
//
class iqtWidgetRadioBoxSubject : public QRadioBox, public ibgWidgetRadioBoxSubject
{

	friend class iggSubjectFactory;

public:

	virtual ~iqtWidgetRadioBoxSubject();

	//
	//  Decorator functions
	//
	virtual int GetValue() const;
	virtual void SetValue(int v);
	virtual void InsertItem(const iString &text, int index = -1);
	virtual void InsertItem(const iImage &icon, const iString &text, int index = -1);
	virtual void SetItem(const iString &text, int index, bool vis);
	virtual void RemoveItem(int index);
	virtual int Count();

protected:

	iqtWidgetRadioBoxSubject(iggWidget *owner, int cols, const iString &title);

	QAbstractButton* FindButton(int index); // porting helper

#ifndef IQT_3
	QButtonGroup *mGroup;
#endif
};

#endif  // IQTWIDGETSELECTIONBOXSUBJECT_H

